@@ -40,8 +40,9 @@ use tensorzero_core::{
     optimization::{
         dicl::UninitializedDiclOptimizationConfig, fireworks_sft::UninitializedFireworksSFTConfig,
         gcp_vertex_gemini_sft::UninitializedGCPVertexGeminiSFTConfig,
-        openai_sft::UninitializedOpenAISFTConfig, together_sft::UninitializedTogetherSFTConfig,
-        OptimizationJobInfoPyClass, OptimizationJobStatus, UninitializedOptimizerInfo,
+        openai_rft::UninitializedOpenAIRFTConfig, openai_sft::UninitializedOpenAISFTConfig,
+        together_sft::UninitializedTogetherSFTConfig, OptimizationJobInfoPyClass,
+        OptimizationJobStatus, UninitializedOptimizerInfo,
     },
     variant::{
         BestOfNSamplingConfigPyClass, ChainOfThoughtConfigPyClass, ChatCompletionConfigPyClass,
@@ -52,10 +53,10 @@ use tensorzero_core::{
     endpoints::{
         datasets::InsertDatapointParams, dynamic_evaluation_run::DynamicEvaluationRunEpisodeParams,
     },
-    gateway_util::ShutdownHandle,
     inference::types::{
         extra_body::UnfilteredInferenceExtraBody, extra_headers::UnfilteredInferenceExtraHeaders,
     },
+    utils::gateway::ShutdownHandle,
 };
 use tensorzero_rust::{
     err_to_http, observability::LogFormat, CacheParamsOptions, Client, ClientBuilder,
@@ -93,6 +94,7 @@ fn tensorzero(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add_class::<LocalHttpGateway>()?;
     m.add_class::<RenderedSample>()?;
     m.add_class::<StoredInference>()?;
+    m.add_class::<UninitializedOpenAIRFTConfig>()?;
     m.add_class::<UninitializedOpenAISFTConfig>()?;
     m.add_class::<UninitializedFireworksSFTConfig>()?;
     m.add_class::<UninitializedDiclOptimizationConfig>()?;
@@ -119,7 +121,7 @@ fn tensorzero(m: &Bound<'_, PyModule>) -> PyResult<()> {
     let json_loads = py_json.getattr("loads")?;
     let json_dumps = py_json.getattr("dumps")?;
 
-    // We don't care if the GILOnceCell was already set
+    // We don't care if the PyOnceLock was already set
     let _ = JSON_LOADS.set(m.py(), json_loads.unbind());
     let _ = JSON_DUMPS.set(m.py(), json_dumps.unbind());
 
@@ -151,18 +153,20 @@ impl LocalHttpGateway {
 }
 
 #[pyfunction]
-#[pyo3(signature = (*, config_file, clickhouse_url, async_setup))]
+#[pyo3(signature = (*, config_file, clickhouse_url, postgres_url, async_setup))]
 fn _start_http_gateway(
     py: Python<'_>,
     config_file: Option<String>,
     clickhouse_url: Option<String>,
+    postgres_url: Option<String>,
     async_setup: bool,
 ) -> PyResult<Bound<'_, PyAny>> {
     warn_no_config(py, config_file.as_deref())?;
     let gateway_fut = async move {
-        let (addr, handle) = tensorzero_core::gateway_util::start_openai_compatible_gateway(
+        let (addr, handle) = tensorzero_core::utils::gateway::start_openai_compatible_gateway(
             config_file,
             clickhouse_url,
+            postgres_url,
         )
         .await?;
         Ok(LocalHttpGateway {
@@ -173,7 +177,7 @@ fn _start_http_gateway(
     if async_setup {
         pyo3_async_runtimes::tokio::future_into_py(py, async move {
             gateway_fut.await.map_err(|e| {
-                Python::with_gil(|py| convert_error(py, TensorZeroError::Other { source: e }))
+                Python::attach(|py| convert_error(py, TensorZeroError::Other { source: e }))
             })
         })
     } else {
@@ -197,7 +201,7 @@ struct AsyncStreamWrapper {
     // after all `AsyncStreamWrapper` objects have been garbage collected.
     // This allows us to safely block from within the Drop impl of `AsyncTensorZeroGateway`.
     // knowing that there are no remaining Python objects holding on to a `ClickhouseConnectionInfo`
-    _gateway: PyObject,
+    _gateway: Py<PyAny>,
 }
 
 #[pymethods]
@@ -223,7 +227,7 @@ impl AsyncStreamWrapper {
             // the `py` parameter from `__anext__`.
             // We need to interact with Python objects here (to build up a Python `InferenceChunk`),
             // so we need the GIL
-            Python::with_gil(|py| {
+            Python::attach(|py| {
                 let chunk = chunk.map_err(|e| convert_error(py, err_to_http(e)))?;
                 parse_inference_chunk(py, chunk)
             })
@@ -239,7 +243,7 @@ struct StreamWrapper {
     // after all `StreamWrapper` objects have been garbage collected.
     // This allows us to safely block from within the Drop impl of `TensorZeroGateway`.
     // knowing that there are no remaining Python objects holding on to a `ClickhouseConnectionInfo`
-    _gateway: PyObject,
+    _gateway: Py<PyAny>,
 }
 
 #[pymethods]
@@ -389,7 +393,7 @@ where
     // our crate (`python`) is the `pymodule` function, rather than
     // a `#[tokio::main]` function, so we need `pyo3_async_runtimes` to keep track of
     // a Tokio runtime for us.
-    py.allow_threads(|| pyo3_async_runtimes::tokio::get_runtime().block_on(fut))
+    py.detach(|| pyo3_async_runtimes::tokio::get_runtime().block_on(fut))
 }
 
 impl BaseTensorZeroGateway {
@@ -629,7 +633,7 @@ impl TensorZeroGateway {
     }
 
     #[classmethod]
-    #[pyo3(signature = (*, config_file=None, clickhouse_url=None, timeout=None))]
+    #[pyo3(signature = (*, config_file=None, clickhouse_url=None, postgres_url=None, timeout=None))]
     /// Initialize the TensorZero client, using an embedded gateway.
     /// This connects to ClickHouse (if provided) and runs DB migrations.
     ///
@@ -641,6 +645,7 @@ impl TensorZeroGateway {
         cls: &Bound<'_, PyType>,
         config_file: Option<&str>,
         clickhouse_url: Option<String>,
+        postgres_url: Option<String>,
         timeout: Option<f64>,
     ) -> PyResult<Py<TensorZeroGateway>> {
         warn_no_config(cls.py(), config_file)?;
@@ -651,6 +656,7 @@ impl TensorZeroGateway {
         let client_fut = ClientBuilder::new(ClientBuilderMode::EmbeddedGateway {
             config_file: config_file.map(PathBuf::from),
             clickhouse_url,
+            postgres_url,
             timeout,
             verify_credentials: true,
             allow_batch_writes: false,
@@ -1209,7 +1215,7 @@ impl AsyncTensorZeroGateway {
             let client = client_fut.await;
             // We need to interact with Python objects here (to build up a Python `AsyncTensorZeroGateway`),
             // so we need the GIL
-            Python::with_gil(|py| {
+            Python::attach(|py| {
                 let client = match client {
                     Ok(client) => client,
                     Err(e) => {
@@ -1274,7 +1280,7 @@ impl AsyncTensorZeroGateway {
     // as `AsyncTensorZeroGateway` would be completely async *except* for this one method
     // (which potentially takes a very long time due to running DB migrations).
     #[classmethod]
-    #[pyo3(signature = (*, config_file=None, clickhouse_url=None, timeout=None, async_setup=true))]
+    #[pyo3(signature = (*, config_file=None, clickhouse_url=None, postgres_url=None, timeout=None, async_setup=true))]
     /// Initialize the TensorZero client, using an embedded gateway.
     /// This connects to ClickHouse (if provided) and runs DB migrations.
     ///
@@ -1288,6 +1294,7 @@ impl AsyncTensorZeroGateway {
         cls: &Bound<'_, PyType>,
         config_file: Option<&str>,
         clickhouse_url: Option<String>,
+        postgres_url: Option<String>,
         timeout: Option<f64>,
         async_setup: bool,
     ) -> PyResult<Py<PyAny>> {
@@ -1299,6 +1306,7 @@ impl AsyncTensorZeroGateway {
         let client_fut = ClientBuilder::new(ClientBuilderMode::EmbeddedGateway {
             config_file: config_file.map(PathBuf::from),
             clickhouse_url,
+            postgres_url,
             timeout,
             verify_credentials: true,
             allow_batch_writes: false,
@@ -1308,7 +1316,7 @@ impl AsyncTensorZeroGateway {
             let client = client_fut.await;
             // We need to interact with Python objects here (to build up a Python `AsyncTensorZeroGateway`),
             // so we need the GIL
-            Python::with_gil(|py| {
+            Python::attach(|py| {
                 let client = match client {
                     Ok(client) => client,
                     Err(e) => {
@@ -1426,7 +1434,7 @@ impl AsyncTensorZeroGateway {
             let res = client.inference(params).await;
             // We need to interact with Python objects here (to build up a Python inference response),
             // so we need the GIL
-            Python::with_gil(|py| {
+            Python::attach(|py| {
                 let output = res.map_err(|e| convert_error(py, e))?;
                 match output {
                     InferenceOutput::NonStreaming(data) => parse_inference_response(py, data),
@@ -1483,7 +1491,7 @@ impl AsyncTensorZeroGateway {
             let res = client.feedback(params).await;
             // We need to interact with Python objects here (to build up a Python feedback response),
             // so we need the GIL
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(resp) => Ok(parse_feedback_response(py, resp)?.into_any()),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1516,7 +1524,7 @@ impl AsyncTensorZeroGateway {
 
         pyo3_async_runtimes::tokio::future_into_py(this.py(), async move {
             let res = client.dynamic_evaluation_run(params).await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(resp) => parse_dynamic_evaluation_run_response(py, resp),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1549,7 +1557,7 @@ impl AsyncTensorZeroGateway {
 
         pyo3_async_runtimes::tokio::future_into_py(this.py(), async move {
             let res = client.dynamic_evaluation_run_episode(run_id, params).await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(resp) => parse_dynamic_evaluation_run_episode_response(py, resp),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1577,7 +1585,7 @@ impl AsyncTensorZeroGateway {
         let uuid = self_module.getattr("UUID")?.unbind();
         pyo3_async_runtimes::tokio::future_into_py(this.py(), async move {
             let res = client.bulk_insert_datapoints(dataset_name, params).await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(uuids) => Ok(PyList::new(
                     py,
                     uuids
@@ -1606,7 +1614,7 @@ impl AsyncTensorZeroGateway {
         let datapoint_id = python_uuid_to_uuid("datapoint_id", datapoint_id)?;
         pyo3_async_runtimes::tokio::future_into_py(this.py(), async move {
             let res = client.delete_datapoint(dataset_name, datapoint_id).await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(()) => Ok(()),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1628,7 +1636,7 @@ impl AsyncTensorZeroGateway {
         let datapoint_id = python_uuid_to_uuid("datapoint_id", datapoint_id)?;
         pyo3_async_runtimes::tokio::future_into_py(this.py(), async move {
             let res = client.get_datapoint(dataset_name, datapoint_id).await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(resp) => Ok(resp.into_py_any(py)?),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1652,7 +1660,7 @@ impl AsyncTensorZeroGateway {
             let res = client
                 .list_datapoints(dataset_name, function_name, limit, offset)
                 .await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(datapoints) => Ok(PyList::new(py, datapoints)?.unbind()),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1724,7 +1732,7 @@ impl AsyncTensorZeroGateway {
                 format: ClickhouseFormat::JsonEachRow,
             };
             let res = client.experimental_list_inferences(params).await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(stored_inferences) => Ok(PyList::new(py, stored_inferences)?.unbind()),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1772,7 +1780,7 @@ impl AsyncTensorZeroGateway {
             let res = client
                 .experimental_render_samples(stored_inferences, variants)
                 .await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(inferences) => Ok(PyList::new(py, inferences)?.unbind()),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1808,7 +1816,7 @@ impl AsyncTensorZeroGateway {
             let res = client
                 .experimental_render_samples(stored_samples, variants)
                 .await;
-            Python::with_gil(|py| match res {
+            Python::attach(|py| match res {
                 Ok(samples) => Ok(PyList::new(py, samples)?.unbind()),
                 Err(e) => Err(convert_error(py, e)),
             })
@@ -1854,7 +1862,7 @@ impl AsyncTensorZeroGateway {
                 .await;
             match res {
                 Ok(job_handle) => Ok(job_handle),
-                Err(e) => Python::with_gil(|py| Err(convert_error(py, e))),
+                Err(e) => Python::attach(|py| Err(convert_error(py, e))),
             }
         })
     }
@@ -1873,7 +1881,7 @@ impl AsyncTensorZeroGateway {
             let res = client.experimental_poll_optimization(&job_handle).await;
             match res {
                 Ok(status) => Ok(OptimizationJobInfoPyClass::new(status)),
-                Err(e) => Python::with_gil(|py| Err(convert_error(py, e))),
+                Err(e) => Python::attach(|py| Err(convert_error(py, e))),
             }
         })
     }
