@@ -3,11 +3,12 @@ use std::time::Duration;
 use std::{collections::HashMap, sync::Arc};
 
 use crate::cache::{
-    embedding_cache_lookup, start_cache_write, CacheData, EmbeddingCacheData,
+    embedding_cache_lookup, start_cache_write, CacheData, CacheValidationInfo, EmbeddingCacheData,
     EmbeddingModelProviderRequest,
 };
 use crate::config::{ProviderTypesConfig, TimeoutsConfig};
 use crate::endpoints::inference::InferenceClients;
+use crate::http::TensorzeroHttpClient;
 use crate::inference::types::extra_body::ExtraBodyConfig;
 use crate::inference::types::RequestMessagesOrBatch;
 use crate::inference::types::{ContentBlock, Text};
@@ -15,6 +16,10 @@ use crate::model::{ModelProviderRequestInfo, UninitializedProviderConfig};
 use crate::model_table::BaseModelTable;
 use crate::model_table::ShorthandModelConfig;
 use crate::providers::azure::AzureProvider;
+use crate::rate_limiting::{
+    get_estimated_tokens, RateLimitResourceUsage, RateLimitedInputContent, RateLimitedRequest,
+    RateLimitedResponse, ScopeInfo,
+};
 use crate::{
     endpoints::inference::InferenceCredentials,
     error::{Error, ErrorDetails, IMPOSSIBLE_ERROR_MESSAGE},
@@ -25,10 +30,10 @@ use crate::{
     providers::openai::OpenAIProvider,
 };
 use futures::future::try_join_all;
-use reqwest::Client;
 use serde::{Deserialize, Serialize};
 use tokio::time::error::Elapsed;
-use tracing::instrument;
+use tracing::{instrument, Span};
+use tracing_futures::Instrument;
 use uuid::Uuid;
 
 #[cfg(any(test, feature = "e2e_tests"))]
@@ -129,8 +134,9 @@ impl EmbeddingModelConfig {
                 })?;
                 let provider_request = EmbeddingModelProviderRequest {
                     request,
-                    provider_name,
                     model_name,
+                    provider_name,
+                    otlp_config: clients.otlp_config,
                 };
                 // TODO: think about how to best handle errors here
                 if clients.cache_options.enabled.read() {
@@ -146,13 +152,11 @@ impl EmbeddingModelConfig {
                         return Ok(cache_lookup);
                     }
                 }
+                let scope_info = ScopeInfo {
+                    tags: &HashMap::default(),
+                };
                 let response = provider_config
-                    .embed(
-                        request,
-                        clients.http_client,
-                        clients.credentials,
-                        &provider_config.into(),
-                    )
+                    .embed(request, clients, &scope_info, &provider_config.into())
                     .await;
 
                 match response {
@@ -168,13 +172,17 @@ impl EmbeddingModelConfig {
                                 let _ = start_cache_write(
                                     clients.clickhouse_connection_info,
                                     provider_request.get_cache_key()?,
-                                    EmbeddingCacheData {
-                                        embedding: float_data.clone(),
+                                    CacheData {
+                                        output: EmbeddingCacheData {
+                                            embedding: float_data.clone(),
+                                        },
+                                        raw_request: response.raw_request.clone(),
+                                        raw_response: response.raw_response.clone(),
+                                        input_tokens: response.usage.input_tokens,
+                                        output_tokens: response.usage.output_tokens,
+                                        finish_reason: None,
                                     },
-                                    &response.raw_request,
-                                    &response.raw_response,
-                                    &response.usage,
-                                    None,
+                                    CacheValidationInfo { tool_config: None },
                                 );
                             }
                         };
@@ -235,6 +243,18 @@ impl EmbeddingInput {
     }
 }
 
+impl RateLimitedInputContent for EmbeddingInput {
+    fn estimated_input_token_usage(&self) -> u64 {
+        match self {
+            EmbeddingInput::Single(text) => get_estimated_tokens(text),
+            EmbeddingInput::Batch(texts) => texts
+                .iter()
+                .map(|text| get_estimated_tokens(text))
+                .sum::<u64>(),
+        }
+    }
+}
+
 impl From<String> for EmbeddingInput {
     fn from(text: String) -> Self {
         EmbeddingInput::Single(text)
@@ -248,6 +268,20 @@ pub struct EmbeddingRequest {
     pub encoding_format: EmbeddingEncodingFormat,
 }
 
+impl RateLimitedRequest for EmbeddingRequest {
+    fn estimated_resource_usage(&self) -> Result<RateLimitResourceUsage, Error> {
+        let EmbeddingRequest {
+            input,
+            dimensions: _,
+            encoding_format: _,
+        } = self;
+        Ok(RateLimitResourceUsage {
+            model_inferences: 1,
+            tokens: input.estimated_input_token_usage(),
+        })
+    }
+}
+
 #[derive(Debug, Clone, Copy)]
 pub struct EmbeddingProviderRequest<'request> {
     pub request: &'request EmbeddingRequest,
@@ -275,6 +309,15 @@ pub struct EmbeddingProviderResponse {
     pub latency: Latency,
 }
 
+impl RateLimitedResponse for EmbeddingProviderResponse {
+    fn resource_usage(&self) -> RateLimitResourceUsage {
+        RateLimitResourceUsage {
+            model_inferences: 1,
+            tokens: self.usage.total_tokens() as u64,
+        }
+    }
+}
+
 #[derive(Debug, PartialEq)]
 pub struct EmbeddingModelResponse {
     pub id: Uuid,
@@ -312,6 +355,21 @@ impl EmbeddingModelResponse {
             cached: true,
         }
     }
+
+    /// We return the actual usage (meaning the number of tokens the user would be billed for)
+    /// in the HTTP response.
+    /// However, we store the number of tokens that would have been used in the database.
+    /// So we need this function to compute the actual usage in order to send it in the HTTP response.
+    pub fn usage_considering_cached(&self) -> Usage {
+        if self.cached {
+            Usage {
+                input_tokens: 0,
+                output_tokens: 0,
+            }
+        } else {
+            self.usage
+        }
+    }
 }
 
 pub struct EmbeddingResponseWithMetadata {
@@ -396,12 +454,11 @@ impl TryFrom<EmbeddingResponseWithMetadata> for ModelInferenceResponseWithMetada
         })
     }
 }
-
 pub trait EmbeddingProvider {
     fn embed(
         &self,
         request: &EmbeddingRequest,
-        client: &Client,
+        client: &TensorzeroHttpClient,
         dynamic_api_keys: &InferenceCredentials,
         model_provider_data: &EmbeddingProviderRequestInfo,
     ) -> impl Future<Output = Result<EmbeddingProviderResponse, Error>> + Send;
@@ -453,33 +510,52 @@ impl From<&EmbeddingProviderRequestInfo> for ModelProviderRequestInfo {
     }
 }
 
-impl EmbeddingProvider for EmbeddingProviderInfo {
-    async fn embed(
+impl EmbeddingProviderInfo {
+    pub async fn embed(
         &self,
         request: &EmbeddingRequest,
-        client: &Client,
-        dynamic_api_keys: &InferenceCredentials,
+        clients: &InferenceClients<'_>,
+        scope_info: &ScopeInfo<'_>,
         model_provider_data: &EmbeddingProviderRequestInfo,
     ) -> Result<EmbeddingProviderResponse, Error> {
-        let response_fut = self
-            .inner
-            .embed(request, client, dynamic_api_keys, model_provider_data);
-        Ok(
-            if let Some(timeout_ms) = self.timeouts.non_streaming.total_ms {
-                let timeout = Duration::from_millis(timeout_ms);
-                tokio::time::timeout(timeout, response_fut)
+        let ticket_borrow = clients
+            .rate_limiting_config
+            .consume_tickets(clients.postgres_connection_info, scope_info, request)
+            .await?;
+        let response_fut = self.inner.embed(
+            request,
+            clients.http_client,
+            clients.credentials,
+            model_provider_data,
+        );
+        let response = if let Some(timeout_ms) = self.timeouts.non_streaming.total_ms {
+            let timeout = Duration::from_millis(timeout_ms);
+            tokio::time::timeout(timeout, response_fut)
+                .await
+                .unwrap_or_else(|_: Elapsed| {
+                    Err(Error::new(ErrorDetails::ModelProviderTimeout {
+                        provider_name: self.provider_name.to_string(),
+                        timeout,
+                        streaming: false,
+                    }))
+                })?
+        } else {
+            response_fut.await?
+        };
+        let postgres_connection_info = clients.postgres_connection_info.clone();
+        let resource_usage = response.resource_usage();
+        tokio::spawn(
+            async move {
+                if let Err(e) = ticket_borrow
+                    .return_tickets(&postgres_connection_info, resource_usage)
                     .await
-                    .unwrap_or_else(|_: Elapsed| {
-                        Err(Error::new(ErrorDetails::ModelProviderTimeout {
-                            provider_name: self.provider_name.to_string(),
-                            timeout,
-                            streaming: false,
-                        }))
-                    })?
-            } else {
-                response_fut.await?
-            },
-        )
+                {
+                    tracing::error!("Failed to return rate limit tickets: {}", e);
+                }
+            }
+            .instrument(Span::current()),
+        );
+        Ok(response)
     }
 }
 
@@ -537,7 +613,7 @@ impl EmbeddingProvider for EmbeddingProviderConfig {
     async fn embed(
         &self,
         request: &EmbeddingRequest,
-        client: &Client,
+        client: &TensorzeroHttpClient,
         dynamic_api_keys: &InferenceCredentials,
         model_provider_data: &EmbeddingProviderRequestInfo,
     ) -> Result<EmbeddingProviderResponse, Error> {
@@ -613,7 +689,7 @@ mod tests {
 
     use crate::{
         cache::{CacheEnabledMode, CacheOptions},
-        db::clickhouse::ClickHouseConnectionInfo,
+        db::{clickhouse::ClickHouseConnectionInfo, postgres::PostgresConnectionInfo},
     };
 
     use super::*;
@@ -659,13 +735,17 @@ mod tests {
                 &request,
                 "fallback",
                 &InferenceClients {
-                    http_client: &Client::new(),
+                    http_client: &TensorzeroHttpClient::new().unwrap(),
+                    clickhouse_connection_info: &ClickHouseConnectionInfo::new_disabled(),
+                    postgres_connection_info: &PostgresConnectionInfo::Disabled,
                     credentials: &InferenceCredentials::default(),
                     cache_options: &CacheOptions {
                         max_age_s: None,
                         enabled: CacheEnabledMode::Off,
                     },
-                    clickhouse_connection_info: &ClickHouseConnectionInfo::new_disabled(),
+                    tags: &Default::default(),
+                    rate_limiting_config: &Default::default(),
+                    otlp_config: &Default::default(),
                 },
             )
             .await;
