@@ -1,4 +1,5 @@
 use std::collections::HashMap;
+use std::sync::Arc;
 use std::time::Duration;
 
 use axum::http::StatusCode;
@@ -13,6 +14,7 @@ use uuid::Uuid;
 
 use crate::inference::types::storage::StoragePath;
 use crate::inference::types::Thought;
+use crate::rate_limiting::{ActiveRateLimitKey, RateLimitingConfigScopes};
 
 /// Controls whether to include raw request/response details in error output
 ///
@@ -100,28 +102,28 @@ impl<T: Debug + Display> Display for DisplayOrDebugGateway<T> {
     }
 }
 
-#[derive(Debug, Error, Serialize)]
+#[derive(Clone, Debug, Error, Serialize)]
 #[cfg_attr(any(test, feature = "e2e_tests"), derive(PartialEq))]
 #[error(transparent)]
 // As long as the struct member is private, we force people to use the `new` method and log the error.
-// We box `ErrorDetails` per the `clippy::result_large_err` lint
-pub struct Error(Box<ErrorDetails>);
+// We arc `ErrorDetails` per the `clippy::result_large_err` lint, as well as to make it cloneable
+pub struct Error(Arc<ErrorDetails>);
 
 impl Error {
     pub fn new(details: ErrorDetails) -> Self {
         details.log();
-        Error(Box::new(details))
+        Error(Arc::new(details))
     }
 
     pub fn new_with_err_logging(details: ErrorDetails, err_logging: bool) -> Self {
         if err_logging {
             details.log();
         }
-        Error(Box::new(details))
+        Error(Arc::new(details))
     }
 
     pub fn new_without_logging(details: ErrorDetails) -> Self {
-        Error(Box::new(details))
+        Error(Arc::new(details))
     }
 
     pub fn status_code(&self) -> StatusCode {
@@ -132,13 +134,13 @@ impl Error {
         &self.0
     }
 
-    pub fn get_owned_details(self) -> ErrorDetails {
-        *self.0
-    }
-
     pub fn log(&self) {
         self.0.log();
     }
+
+    pub fn is_retryable(&self) -> bool {
+        self.0.is_retryable()
+    }
 }
 
 // Expect for derive Serialize
@@ -181,6 +183,7 @@ pub enum ErrorDetails {
     },
     ApiKeyMissing {
         provider_name: String,
+        message: String,
     },
     AppState {
         message: String,
@@ -238,9 +241,16 @@ pub enum ErrorDetails {
         dataset_name: String,
         datapoint_id: Uuid,
     },
+    DiclMissingOutput,
     DuplicateTool {
         name: String,
     },
+    DuplicateRateLimitingConfigScope {
+        scope: RateLimitingConfigScopes,
+    },
+    DynamicEndpointNotFound {
+        key_name: String,
+    },
     DynamicJsonSchema {
         message: String,
     },
@@ -282,6 +292,9 @@ pub enum ErrorDetails {
     InvalidDynamicTemplatePath {
         name: String,
     },
+    InvalidDynamicEndpoint {
+        url: String,
+    },
     InvalidEncodedJobHandle,
     InvalidJobHandle {
         message: String,
@@ -439,9 +452,32 @@ pub enum ErrorDetails {
     OutputValidation {
         source: Box<Error>,
     },
+    // TODO(shuyang): once 3496 merges: change all of these to taking sqlx errors rather than string. Note also sqlx::Error is not Serialize?
+    PostgresConnectionInitialization {
+        message: String,
+    },
+    PostgresConnection {
+        message: String,
+    },
+    PostgresMigration {
+        message: String,
+    },
+    PostgresQuery {
+        function_name: Option<String>,
+        message: String,
+    },
+    PostgresResult {
+        result_type: &'static str,
+        message: String,
+    },
     ProviderNotFound {
         provider_name: String,
     },
+    RateLimitExceeded {
+        key: ActiveRateLimitKey,
+        tickets_remaining: u64,
+    },
+    RateLimitMissingMaxTokens,
     Serialization {
         message: String,
     },
@@ -540,8 +576,11 @@ impl ErrorDetails {
             ErrorDetails::ObjectStoreWrite { .. } => tracing::Level::ERROR,
             ErrorDetails::Config { .. } => tracing::Level::ERROR,
             ErrorDetails::DatapointNotFound { .. } => tracing::Level::WARN,
+            ErrorDetails::DiclMissingOutput => tracing::Level::ERROR,
             ErrorDetails::DuplicateTool { .. } => tracing::Level::WARN,
+            ErrorDetails::DuplicateRateLimitingConfigScope { .. } => tracing::Level::WARN,
             ErrorDetails::DynamicJsonSchema { .. } => tracing::Level::WARN,
+            ErrorDetails::DynamicEndpointNotFound { .. } => tracing::Level::WARN,
             ErrorDetails::FileRead { .. } => tracing::Level::ERROR,
             ErrorDetails::GCPCredentials { .. } => tracing::Level::ERROR,
             ErrorDetails::Inference { .. } => tracing::Level::ERROR,
@@ -580,6 +619,7 @@ impl ErrorDetails {
             ErrorDetails::InvalidTemplatePath => tracing::Level::ERROR,
             ErrorDetails::InvalidTool { .. } => tracing::Level::ERROR,
             ErrorDetails::InvalidUuid { .. } => tracing::Level::ERROR,
+            ErrorDetails::InvalidDynamicEndpoint { .. } => tracing::Level::WARN,
             ErrorDetails::InvalidValFraction { .. } => tracing::Level::WARN,
             ErrorDetails::JsonRequest { .. } => tracing::Level::WARN,
             ErrorDetails::JsonSchema { .. } => tracing::Level::ERROR,
@@ -599,6 +639,13 @@ impl ErrorDetails {
             ErrorDetails::OutputValidation { .. } => tracing::Level::WARN,
             ErrorDetails::OptimizationResponse { .. } => tracing::Level::ERROR,
             ErrorDetails::ProviderNotFound { .. } => tracing::Level::ERROR,
+            ErrorDetails::PostgresConnectionInitialization { .. } => tracing::Level::ERROR,
+            ErrorDetails::PostgresConnection { .. } => tracing::Level::ERROR,
+            ErrorDetails::PostgresMigration { .. } => tracing::Level::ERROR,
+            ErrorDetails::PostgresResult { .. } => tracing::Level::ERROR,
+            ErrorDetails::PostgresQuery { .. } => tracing::Level::ERROR,
+            ErrorDetails::RateLimitExceeded { .. } => tracing::Level::WARN,
+            ErrorDetails::RateLimitMissingMaxTokens => tracing::Level::WARN,
             ErrorDetails::Serialization { .. } => tracing::Level::ERROR,
             ErrorDetails::StreamError { .. } => tracing::Level::ERROR,
             ErrorDetails::ToolNotFound { .. } => tracing::Level::WARN,
@@ -642,8 +689,11 @@ impl ErrorDetails {
             ErrorDetails::ObjectStoreUnconfigured { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::DatapointNotFound { .. } => StatusCode::NOT_FOUND,
             ErrorDetails::Config { .. } => StatusCode::INTERNAL_SERVER_ERROR,
+            ErrorDetails::DiclMissingOutput => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::DuplicateTool { .. } => StatusCode::BAD_REQUEST,
+            ErrorDetails::DuplicateRateLimitingConfigScope { .. } => StatusCode::BAD_REQUEST,
             ErrorDetails::DynamicJsonSchema { .. } => StatusCode::BAD_REQUEST,
+            ErrorDetails::DynamicEndpointNotFound { .. } => StatusCode::NOT_FOUND,
             ErrorDetails::FileRead { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::GCPCredentials { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::InvalidInferenceTarget { .. } => StatusCode::BAD_REQUEST,
@@ -674,6 +724,7 @@ impl ErrorDetails {
             ErrorDetails::InvalidCandidate { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::InvalidDiclConfig { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::InvalidDatasetName { .. } => StatusCode::BAD_REQUEST,
+            ErrorDetails::InvalidDynamicEndpoint { .. } => StatusCode::BAD_REQUEST,
             ErrorDetails::InvalidDynamicEvaluationRun { .. } => StatusCode::BAD_REQUEST,
             ErrorDetails::InvalidDynamicTemplatePath { .. } => StatusCode::BAD_REQUEST,
             ErrorDetails::InvalidFunctionVariants { .. } => StatusCode::INTERNAL_SERVER_ERROR,
@@ -707,6 +758,15 @@ impl ErrorDetails {
             ErrorDetails::OutputParsing { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::OutputValidation { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::ProviderNotFound { .. } => StatusCode::NOT_FOUND,
+            ErrorDetails::PostgresConnectionInitialization { .. } => {
+                StatusCode::INTERNAL_SERVER_ERROR
+            }
+            ErrorDetails::PostgresConnection { .. } => StatusCode::INTERNAL_SERVER_ERROR,
+            ErrorDetails::PostgresQuery { .. } => StatusCode::INTERNAL_SERVER_ERROR,
+            ErrorDetails::PostgresResult { .. } => StatusCode::INTERNAL_SERVER_ERROR,
+            ErrorDetails::PostgresMigration { .. } => StatusCode::INTERNAL_SERVER_ERROR,
+            ErrorDetails::RateLimitExceeded { .. } => StatusCode::TOO_MANY_REQUESTS,
+            ErrorDetails::RateLimitMissingMaxTokens => StatusCode::BAD_REQUEST,
             ErrorDetails::Serialization { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::StreamError { .. } => StatusCode::INTERNAL_SERVER_ERROR,
             ErrorDetails::ToolNotFound { .. } => StatusCode::BAD_REQUEST,
@@ -745,6 +805,17 @@ impl ErrorDetails {
             tracing::Level::TRACE => tracing::trace!("{self}"),
         }
     }
+
+    pub fn is_retryable(&self) -> bool {
+        match &self {
+            ErrorDetails::RateLimitExceeded { .. } => false,
+            // For ModelProvidersExhausted we will retry if any provider error is retryable
+            ErrorDetails::ModelProvidersExhausted { provider_errors } => provider_errors
+                .iter()
+                .any(|(_, error)| error.is_retryable()),
+            _ => true,
+        }
+    }
 }
 
 impl std::fmt::Display for ErrorDetails {
@@ -834,8 +905,11 @@ impl std::fmt::Display for ErrorDetails {
             ErrorDetails::Glob { glob, message } => {
                 write!(f, "Error using glob: `{glob}`: {message}")
             }
-            ErrorDetails::ApiKeyMissing { provider_name } => {
-                write!(f, "API key missing for provider: {provider_name}")
+            ErrorDetails::ApiKeyMissing {
+                provider_name,
+                message,
+            } => {
+                write!(f, "API key missing for provider {provider_name}: {message}")
             }
             ErrorDetails::AppState { message } => {
                 write!(f, "Error initializing AppState: {message}")
@@ -888,15 +962,25 @@ impl std::fmt::Display for ErrorDetails {
                     "Datapoint not found for dataset: {dataset_name} and id: {datapoint_id}"
                 )
             }
+            ErrorDetails::DiclMissingOutput => {
+                write!(f, "DICL example missing output. There was a bug in a notebook from 2025-08 that may have caused the output to not be written to ClickHouse. You can remove the examples with missing output by running the query `DELETE FROM DynamicInContextLearningExample WHERE empty(output)`.")
+            }
             ErrorDetails::DuplicateTool { name } => {
                 write!(f, "Duplicate tool name: {name}. Tool names must be unique.")
             }
+            ErrorDetails::DuplicateRateLimitingConfigScope { scope } => {
+                write!(f, "Duplicate rate limiting config scope: {scope:?}. Rate limiting config scopes must be unique.")
+            }
             ErrorDetails::DynamicJsonSchema { message } => {
                 write!(
                     f,
                     "Error in compiling client-provided JSON schema: {message}"
                 )
             }
+            ErrorDetails::DynamicEndpointNotFound { key_name } => {
+                write!(f, "Dynamic endpoint '{key_name}' not found in credentials")
+            }
+
             ErrorDetails::FileRead { message, file_path } => {
                 write!(f, "Error reading file {file_path}: {message}")
             }
@@ -999,6 +1083,9 @@ impl std::fmt::Display for ErrorDetails {
                     "Dynamic evaluation run not found for episode id: {episode_id}",
                 )
             }
+            ErrorDetails::InvalidDynamicEndpoint { url } => {
+                write!(f, "Invalid dynamic endpoint URL: {url}")
+            }
             ErrorDetails::InvalidDynamicTemplatePath { name } => {
                 write!(f, "Invalid dynamic template path: {name}. There is likely a duplicate template in the config.")
             }
@@ -1159,9 +1246,58 @@ impl std::fmt::Display for ErrorDetails {
             ErrorDetails::OutputValidation { source } => {
                 write!(f, "Output validation failed with messages: {source}")
             }
+            ErrorDetails::PostgresConnectionInitialization { message } => {
+                write!(
+                    f,
+                    "Postgres connection initialization failed with message: {message}"
+                )
+            }
+            ErrorDetails::PostgresConnection { message } => {
+                write!(f, "Error connecting to Postgres: {message}")
+            }
+            ErrorDetails::PostgresMigration { message } => {
+                write!(f, "Postgres migration failed with message: {message}")
+            }
+            ErrorDetails::PostgresResult {
+                result_type,
+                message,
+            } => {
+                write!(
+                    f,
+                    "Unexpected Postgres result of type {result_type}: {message}"
+                )
+            }
+            ErrorDetails::PostgresQuery {
+                function_name,
+                message,
+            } => match function_name {
+                Some(function_name) => write!(
+                    f,
+                    "Postgres query failed in function {function_name} with message: {message}"
+                ),
+                None => write!(f, "Postgres query failed: {message}"),
+            },
             ErrorDetails::ProviderNotFound { provider_name } => {
                 write!(f, "Provider not found: {provider_name}")
             }
+            ErrorDetails::RateLimitExceeded {
+                key,
+                tickets_remaining,
+            } => {
+                // TODO: improve this error:
+                // - `{key}` should more closely match the definition of the rule in TOML
+                // - Display the number of requested tickets (units) if possible.
+                write!(
+                    f,
+                    "TensorZero rate limit exceeded for rule {key}. {tickets_remaining} units currently available."
+                )
+            }
+            ErrorDetails::RateLimitMissingMaxTokens => {
+                write!(
+                    f,
+                    "Missing `max_tokens` for request subject to rate limiting rules."
+                )
+            }
             ErrorDetails::StreamError { source } => {
                 write!(f, "Error in streaming response: {source}")
             }
