@@ -37,7 +37,7 @@ lazy_static! {
     /// The URL to send usage data to.
     /// Configurable via the `TENSORZERO_HOWDY_URL` environment variable for testing.
     pub static ref HOWDY_URL: String =
-        env::var("TENSORZERO_HOWDY_URL").unwrap_or("https://howdy.tensorzero.com".to_string());
+        env::var("TENSORZERO_HOWDY_URL").unwrap_or_else(|_| "https://howdy.tensorzero.com".to_string());
 }
 
 /// Setup the howdy loop.
