@@ -3,18 +3,20 @@ use std::borrow::Cow;
 use pyo3::exceptions::{PyRuntimeError, PyValueError};
 use pyo3::types::{IntoPyDict, PyDict};
 use pyo3::{intern, prelude::*};
-use pyo3::{sync::GILOnceCell, types::PyModule, Bound, Py, PyAny, PyErr, PyResult, Python};
+use pyo3::{sync::PyOnceLock, types::PyModule, Bound, Py, PyAny, PyErr, PyResult, Python};
 use serde::Deserialize;
 use serde_json::Value;
 use uuid::Uuid;
 
 use crate::endpoints::datasets::Datapoint;
 use crate::inference::types::stored_input::StoredInput;
+use crate::inference::types::ResolvedContentBlock;
 use crate::inference::types::{
     stored_input::StoredInputMessageContent, ContentBlockChatOutput, ResolvedInputMessageContent,
 };
 use crate::optimization::dicl::UninitializedDiclOptimizationConfig;
 use crate::optimization::fireworks_sft::UninitializedFireworksSFTConfig;
+use crate::optimization::openai_rft::UninitializedOpenAIRFTConfig;
 use crate::optimization::openai_sft::UninitializedOpenAISFTConfig;
 use crate::optimization::together_sft::UninitializedTogetherSFTConfig;
 use crate::optimization::UninitializedOptimizerConfig;
@@ -23,13 +25,11 @@ use crate::stored_inference::{
 };
 use pyo3::types::PyNone;
 
-use super::ContentBlock;
-
-pub static JSON_LOADS: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
-pub static JSON_DUMPS: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
-pub static UUID_UUID: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
-static TENSORZERO_INTERNAL_ERROR: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
-static TENSORZERO_ERROR: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+pub static JSON_LOADS: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+pub static JSON_DUMPS: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+pub static UUID_UUID: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+static TENSORZERO_INTERNAL_ERROR: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+static TENSORZERO_ERROR: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
 
 pub fn uuid_to_python(py: Python<'_>, uuid: Uuid) -> PyResult<Bound<'_, PyAny>> {
     let uuid_class = UUID_UUID.get_or_try_init::<_, PyErr>(py, || {
@@ -41,12 +41,24 @@ pub fn uuid_to_python(py: Python<'_>, uuid: Uuid) -> PyResult<Bound<'_, PyAny>>
     Ok(uuid_obj.into_bound(py))
 }
 
+fn import_template_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
+    // NOTE: we are reusing the type as is used in our output.
+    // We may want to consider not doing this so that we don't have these tied together in our interface.
+    // However, they are currently nearly identical so this would be duplicated code for now and
+    // not intutitive for users
+    static TEMPLATE_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+    TEMPLATE_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
+        let self_module = PyModule::import(py, "tensorzero.types")?;
+        Ok(self_module.getattr("Template")?.unbind())
+    })
+}
+
 fn import_text_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
     // NOTE: we are reusing the type as is used in our output.
     // We may want to consider not doing this so that we don't have these tied together in our interface.
     // However, they are currently nearly identical so this would be duplicated code for now and
     // not intutitive for users
-    static TEXT_CONTENT_BLOCK: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+    static TEXT_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     TEXT_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
         let self_module = PyModule::import(py, "tensorzero.types")?;
         Ok(self_module.getattr("Text")?.unbind())
@@ -54,7 +66,7 @@ fn import_text_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
 }
 
 fn import_raw_text_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
-    static RAW_TEXT_CONTENT_BLOCK: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+    static RAW_TEXT_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     RAW_TEXT_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
         let self_module = PyModule::import(py, "tensorzero.types")?;
         Ok(self_module.getattr("RawText")?.unbind())
@@ -62,7 +74,7 @@ fn import_raw_text_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
 }
 
 fn import_file_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
-    static FILE_CONTENT_BLOCK: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+    static FILE_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     FILE_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
         let self_module = PyModule::import(py, "tensorzero.types")?;
         Ok(self_module.getattr("FileBase64")?.unbind())
@@ -70,7 +82,7 @@ fn import_file_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
 }
 
 fn import_tool_call_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
-    static TOOL_CALL_CONTENT_BLOCK: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+    static TOOL_CALL_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     TOOL_CALL_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
         let self_module = PyModule::import(py, "tensorzero.types")?;
         Ok(self_module.getattr("ToolCall")?.unbind())
@@ -78,7 +90,7 @@ fn import_tool_call_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
 }
 
 fn import_thought_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
-    static THOUGHT_CONTENT_BLOCK: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+    static THOUGHT_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     THOUGHT_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
         let self_module = PyModule::import(py, "tensorzero.types")?;
         Ok(self_module.getattr("Thought")?.unbind())
@@ -86,7 +98,7 @@ fn import_thought_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
 }
 
 fn import_tool_result_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
-    static TOOL_RESULT_CONTENT_BLOCK: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+    static TOOL_RESULT_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     TOOL_RESULT_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
         let self_module = PyModule::import(py, "tensorzero.types")?;
         Ok(self_module.getattr("ToolResult")?.unbind())
@@ -94,30 +106,30 @@ fn import_tool_result_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
 }
 
 fn import_unknown_content_block(py: Python<'_>) -> PyResult<&Py<PyAny>> {
-    static UNKNOWN_CONTENT_BLOCK: GILOnceCell<Py<PyAny>> = GILOnceCell::new();
+    static UNKNOWN_CONTENT_BLOCK: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     UNKNOWN_CONTENT_BLOCK.get_or_try_init::<_, PyErr>(py, || {
         let self_module = PyModule::import(py, "tensorzero.types")?;
         Ok(self_module.getattr("UnknownContentBlock")?.unbind())
     })
 }
 
-pub fn content_block_to_python(
+pub fn resolved_content_block_to_python(
     py: Python<'_>,
-    content_block: &ContentBlock,
+    content_block: &ResolvedContentBlock,
 ) -> PyResult<Py<PyAny>> {
     match content_block {
-        ContentBlock::Text(text) => {
+        ResolvedContentBlock::Text(text) => {
             let text_content_block = import_text_content_block(py)?;
             text_content_block.call1(py, (text.text.clone(),))
         }
-        ContentBlock::File(file) => {
+        ResolvedContentBlock::File(file) => {
             let file_content_block = import_file_content_block(py)?;
             file_content_block.call1(
                 py,
                 (file.file.data.clone(), file.file.mime_type.to_string()),
             )
         }
-        ContentBlock::ToolCall(tool_call) => {
+        ResolvedContentBlock::ToolCall(tool_call) => {
             let tool_call_content_block = import_tool_call_content_block(py)?;
             tool_call_content_block.call1(
                 py,
@@ -130,11 +142,11 @@ pub fn content_block_to_python(
                 ),
             )
         }
-        ContentBlock::Thought(thought) => {
+        ResolvedContentBlock::Thought(thought) => {
             let thought_content_block = import_thought_content_block(py)?;
             thought_content_block.call1(py, (thought.text.clone(),))
         }
-        ContentBlock::ToolResult(tool_result) => {
+        ResolvedContentBlock::ToolResult(tool_result) => {
             let tool_result_content_block = import_tool_result_content_block(py)?;
             tool_result_content_block.call1(
                 py,
@@ -145,7 +157,7 @@ pub fn content_block_to_python(
                 ),
             )
         }
-        ContentBlock::Unknown {
+        ResolvedContentBlock::Unknown {
             data,
             model_provider_name,
         } => {
@@ -212,6 +224,11 @@ pub fn stored_input_message_content_to_python(
                 }
             }
         }
+        StoredInputMessageContent::Template(template) => {
+            let template_content_block = import_template_content_block(py)?;
+            let arguments_py = serialize_to_dict(py, template.arguments)?;
+            template_content_block.call1(py, (template.name, arguments_py))
+        }
         StoredInputMessageContent::ToolCall(tool_call) => {
             let tool_call_content_block = import_tool_call_content_block(py)?;
             let parsed_arguments_py = JSON_LOADS
@@ -267,19 +284,15 @@ pub fn resolved_input_message_content_to_python(
     content: ResolvedInputMessageContent,
 ) -> PyResult<Py<PyAny>> {
     match content {
-        ResolvedInputMessageContent::Text { value } => {
+        ResolvedInputMessageContent::Text { text } => {
             let text_content_block = import_text_content_block(py)?;
-            match value {
-                Value::String(s) => {
-                    let kwargs = [(intern!(py, "text"), s)].into_py_dict(py)?;
-                    text_content_block.call(py, (), Some(&kwargs))
-                }
-                _ => {
-                    let value = serialize_to_dict(py, value)?;
-                    let kwargs = [(intern!(py, "arguments"), value)].into_py_dict(py)?;
-                    text_content_block.call(py, (), Some(&kwargs))
-                }
-            }
+            let kwargs = [(intern!(py, "text"), text)].into_py_dict(py)?;
+            text_content_block.call(py, (), Some(&kwargs))
+        }
+        ResolvedInputMessageContent::Template(template) => {
+            let template_content_block = import_template_content_block(py)?;
+            let arguments_py = serialize_to_dict(py, template.arguments)?;
+            template_content_block.call1(py, (template.name, arguments_py))
         }
         ResolvedInputMessageContent::ToolCall(tool_call) => {
             let tool_call_content_block = import_tool_call_content_block(py)?;
@@ -390,6 +403,8 @@ pub fn deserialize_optimization_config(
 ) -> PyResult<UninitializedOptimizerConfig> {
     if obj.is_instance_of::<UninitializedOpenAISFTConfig>() {
         Ok(UninitializedOptimizerConfig::OpenAISFT(obj.extract()?))
+    } else if obj.is_instance_of::<UninitializedOpenAIRFTConfig>() {
+        Ok(UninitializedOptimizerConfig::OpenAIRFT(obj.extract()?))
     } else if obj.is_instance_of::<UninitializedFireworksSFTConfig>() {
         Ok(UninitializedOptimizerConfig::FireworksSFT(obj.extract()?))
     } else if obj.is_instance_of::<UninitializedTogetherSFTConfig>() {
@@ -400,7 +415,7 @@ pub fn deserialize_optimization_config(
         Ok(UninitializedOptimizerConfig::Dicl(obj.extract()?))
     } else {
         Err(PyValueError::new_err(
-            "Invalid optimization config. Expected OpenAISFTConfig, FireworksSFTConfig, TogetherSFTConfig, or DiclOptimizationConfig",
+            "Invalid optimization config. Expected OpenAISFTConfig, OpenAIRFTConfig, FireworksSFTConfig, TogetherSFTConfig, or DiclOptimizationConfig",
         ))
     }
 }
