@@ -1,5 +1,5 @@
 use minijinja::{Environment, UndefinedBehavior};
-use serde_json::Value;
+use serde::Serialize;
 use std::{
     collections::{HashMap, HashSet},
     path::Path,
@@ -69,7 +69,11 @@ impl TemplateConfig<'_> {
     }
 
     // Templates a message with a MiniJinja template.
-    pub fn template_message(&self, template_name: &str, context: &Value) -> Result<String, Error> {
+    pub fn template_message<S: Serialize>(
+        &self,
+        template_name: &str,
+        context: &S,
+    ) -> Result<String, Error> {
         let template = self.env.get_template(template_name).map_err(|_| {
             Error::new(ErrorDetails::MiniJinjaTemplateMissing {
                 template_name: template_name.to_string(),
@@ -206,7 +210,9 @@ const MIXTURE_OF_N_FUSER_CANDIDATES: &str = r"Here are the candidate answers (wi
 
 #[cfg(test)]
 pub(crate) mod tests {
-    use crate::jsonschema_util::StaticJSONSchema;
+    use std::path::PathBuf;
+
+    use crate::{config::path::ResolvedTomlPath, jsonschema_util::StaticJSONSchema};
 
     use super::*;
     use serde_json::json;
@@ -333,6 +339,52 @@ pub(crate) mod tests {
         .unwrap()
     }
 
+    // Filled in system template
+    pub fn get_system_filled_template() -> ResolvedTomlPath {
+        ResolvedTomlPath::new_for_tests(
+            PathBuf::from("system_filled"),
+            Some("You are a helpful and friendly assistant named ChatGPT".to_string()),
+        )
+    }
+
+    // Filled in user template
+    pub fn get_user_filled_template() -> ResolvedTomlPath {
+        ResolvedTomlPath::new_for_tests(
+            PathBuf::from("user_filled"),
+            Some("What's the capital of Japan?".to_string()),
+        )
+    }
+
+    // Filled in assistant template
+    pub fn get_assistant_filled_template() -> ResolvedTomlPath {
+        ResolvedTomlPath::new_for_tests(
+            PathBuf::from("assistant_filled"),
+            Some("I'm sorry but I can't help you with that because of it's against my ethical guidelines".to_string()),
+        )
+    }
+
+    // System template
+    pub fn get_system_template() -> ResolvedTomlPath {
+        ResolvedTomlPath::new_for_tests(
+            PathBuf::from("system"),
+            Some("You are a helpful and friendly assistant named {{ assistant_name }}".to_string()),
+        )
+    }
+
+    pub fn get_assistant_template() -> ResolvedTomlPath {
+        ResolvedTomlPath::new_for_tests(
+            PathBuf::from("assistant"),
+            Some("I'm sorry but I can't help you with that because of {{ reason }}".to_string()),
+        )
+    }
+
+    pub fn get_greeting_with_age_template() -> ResolvedTomlPath {
+        ResolvedTomlPath::new_for_tests(
+            PathBuf::from("greeting_with_age"),
+            Some("Hello, {{ name }}! You are {{ age }} years old.".to_string()),
+        )
+    }
+
     pub fn get_test_template_config<'a>() -> TemplateConfig<'a> {
         let mut templates = HashMap::new();
 
@@ -342,37 +394,34 @@ pub(crate) mod tests {
         // Template 2
         templates.insert(
             "greeting_with_age".to_string(),
-            "Hello, {{ name }}! You are {{ age }} years old.".to_string(),
+            get_greeting_with_age_template().read().unwrap(),
         );
 
         // System template
-        templates.insert(
-            "system".to_string(),
-            "You are a helpful and friendly assistant named {{ assistant_name }}".to_string(),
-        );
+        templates.insert("system".to_string(), get_system_template().read().unwrap());
 
         // Filled in system template
         templates.insert(
             "system_filled".to_string(),
-            "You are a helpful and friendly assistant named ChatGPT".to_string(),
+            get_system_filled_template().read().unwrap(),
         );
 
         // Assistant Template
         templates.insert(
             "assistant".to_string(),
-            "I'm sorry but I can't help you with that because of {{ reason }}".to_string(),
+            get_assistant_template().read().unwrap(),
         );
 
         // Filled in assistant template
         templates.insert(
             "assistant_filled".to_string(),
-            "I'm sorry but I can't help you with that because of it's against my ethical guidelines".to_string(),
+            get_assistant_filled_template().read().unwrap(),
         );
 
         // Filled in user template
         templates.insert(
             "user_filled".to_string(),
-            "What's the capital of Japan?".to_string(),
+            get_user_filled_template().read().unwrap(),
         );
 
         // Template with tojson filter
