@@ -11,8 +11,8 @@ use aws_sdk_bedrockruntime::types::{
 };
 use aws_smithy_types::error::display::DisplayErrorContext;
 use aws_types::region::Region;
+use futures::future::try_join_all;
 use futures::StreamExt;
-use itertools::Itertools;
 use reqwest::StatusCode;
 use serde::Serialize;
 use std::time::Duration;
@@ -24,6 +24,7 @@ use super::helpers::peek_first_chunk;
 use crate::cache::ModelProviderRequest;
 use crate::endpoints::inference::InferenceCredentials;
 use crate::error::{DisplayOrDebugGateway, Error, ErrorDetails};
+use crate::http::TensorzeroHttpClient;
 use crate::inference::types::batch::BatchRequestRow;
 use crate::inference::types::batch::PollBatchInferenceResponse;
 use crate::inference::types::file::mime_type_to_ext;
@@ -80,20 +81,20 @@ impl InferenceProvider for AWSBedrockProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<ProviderInferenceResponse, Error> {
         // TODO (#55): add support for guardrails and additional fields
 
-        let mut messages: Vec<Message> = request
-            .messages
-            .iter()
-            .map(Message::try_from)
-            .filter_ok(|m| !m.content.is_empty())
-            .collect::<Result<Vec<_>, _>>()?;
-
+        let mut messages: Vec<Message> =
+            try_join_all(request.messages.iter().map(message_from_request_message))
+                .await?
+                .into_iter()
+                .filter(|m| !m.content.is_empty())
+                .collect();
         if self.model_id.contains("claude")
             && request.function_type == FunctionType::Json
             && matches!(
@@ -101,7 +102,7 @@ impl InferenceProvider for AWSBedrockProvider {
                 ModelInferenceRequestJsonMode::On | ModelInferenceRequestJsonMode::Strict
             )
         {
-            prefill_json_message(&mut messages)?;
+            prefill_json_message(&mut messages).await?;
         }
 
         let mut inference_config = InferenceConfiguration::builder();
@@ -172,10 +173,16 @@ impl InferenceProvider for AWSBedrockProvider {
             get_raw_response,
         } = build_interceptor(request, model_provider, model_name.to_string());
 
+        // We need to use the `aws_http_client::Client` wrapper type, which currently
+        // doesn't work with the `TensorzeroHttpClient` type.
+        // This causes us to lose out on things like connection pooling and outgoing OTEL headers.
+        // TODO: make this use `TensorzeroHttpClient`
         let new_config = self
             .base_config
             .clone()
-            .http_client(super::aws_http_client::Client::new(http_client.clone()));
+            .http_client(super::aws_http_client::Client::new(
+                http_client.dangerous_get_fallback_client().clone(),
+            ));
         let start_time = Instant::now();
         let output = bedrock_request
             .customize()
@@ -222,18 +229,19 @@ impl InferenceProvider for AWSBedrockProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<(PeekableProviderInferenceResponseStream, String), Error> {
         // TODO (#55): add support for guardrails and additional fields
 
-        let mut messages: Vec<Message> = request
-            .messages
-            .iter()
-            .map(Message::try_from)
-            .collect::<Result<Vec<_>, _>>()?;
+        let mut messages: Vec<Message> =
+            try_join_all(request.messages.iter().map(message_from_request_message))
+                .await?
+                .into_iter()
+                .collect();
 
         if self.model_id.contains("claude")
             && request.function_type == FunctionType::Json
@@ -242,7 +250,7 @@ impl InferenceProvider for AWSBedrockProvider {
                 ModelInferenceRequestJsonMode::On | ModelInferenceRequestJsonMode::Strict
             )
         {
-            prefill_json_message(&mut messages)?;
+            prefill_json_message(&mut messages).await?;
         }
 
         let mut inference_config = InferenceConfiguration::builder();
@@ -312,10 +320,17 @@ impl InferenceProvider for AWSBedrockProvider {
             get_raw_request,
             get_raw_response,
         } = build_interceptor(request, model_provider, model_name.to_string());
+
+        // We need to use the `aws_http_client::Client` wrapper type, which currently
+        // doesn't work with the `TensorzeroHttpClient` type.
+        // This causes us to lose out on things like connection pooling and outgoing OTEL headers.
+        // TODO: make this use `TensorzeroHttpClient`
         let new_config = self
             .base_config
             .clone()
-            .http_client(super::aws_http_client::Client::new(http_client.clone()));
+            .http_client(super::aws_http_client::Client::new(
+                http_client.dangerous_get_fallback_client().clone(),
+            ));
 
         let start_time = Instant::now();
         let stream = bedrock_request
@@ -356,7 +371,7 @@ impl InferenceProvider for AWSBedrockProvider {
     async fn start_batch_inference<'a>(
         &'a self,
         _requests: &'a [ModelInferenceRequest<'_>],
-        _client: &'a reqwest::Client,
+        _client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<StartBatchProviderInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -368,7 +383,7 @@ impl InferenceProvider for AWSBedrockProvider {
     async fn poll_batch_inference<'a>(
         &'a self,
         _batch_request: &'a BatchRequestRow<'a>,
-        _http_client: &'a reqwest::Client,
+        _http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<PollBatchInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -584,175 +599,177 @@ impl From<Role> for ConversationRole {
 }
 
 /// Prefill messages for AWS Bedrock when conditions are met
-fn prefill_json_message(messages: &mut Vec<Message>) -> Result<(), Error> {
+async fn prefill_json_message(messages: &mut Vec<Message>) -> Result<(), Error> {
     // Add a JSON-prefill message for AWS Bedrock's JSON mode
-    messages.push(Message::try_from(&RequestMessage {
-        role: Role::Assistant,
-        content: vec![ContentBlock::Text(Text {
-            text: "Here is the JSON requested:\n{".to_string(),
-        })],
-    })?);
+    messages.push(
+        message_from_request_message(&RequestMessage {
+            role: Role::Assistant,
+            content: vec![ContentBlock::Text(Text {
+                text: "Here is the JSON requested:\n{".to_string(),
+            })],
+        })
+        .await?,
+    );
     Ok(())
 }
 
-impl TryFrom<&ContentBlock> for Option<BedrockContentBlock> {
-    type Error = Error;
+async fn bedrock_content_block_from_content_block(
+    block: &ContentBlock,
+) -> Result<Option<BedrockContentBlock>, Error> {
+    match block {
+        ContentBlock::Text(Text { text }) => Ok(Some(BedrockContentBlock::Text(text.clone()))),
+        ContentBlock::ToolCall(tool_call) => {
+            // Convert the tool call arguments from String to JSON Value...
+            let input = serde_json::from_str(&tool_call.arguments).map_err(|e| {
+                Error::new(ErrorDetails::InferenceClient {
+                    raw_request: None,
+                    raw_response: Some(tool_call.arguments.clone()),
+                    status_code: Some(StatusCode::BAD_REQUEST),
+                    message: format!(
+                        "Error parsing tool call arguments as JSON Value: {}",
+                        DisplayOrDebugGateway::new(e)
+                    ),
+                    provider_type: PROVIDER_TYPE.to_string(),
+                })
+            })?;
 
-    fn try_from(block: &ContentBlock) -> Result<Self, Self::Error> {
-        match block {
-            ContentBlock::Text(Text { text }) => Ok(Some(BedrockContentBlock::Text(text.clone()))),
-            ContentBlock::ToolCall(tool_call) => {
-                // Convert the tool call arguments from String to JSON Value...
-                let input = serde_json::from_str(&tool_call.arguments).map_err(|e| {
-                    Error::new(ErrorDetails::InferenceClient {
-                        raw_request: None,
-                        raw_response: Some(tool_call.arguments.clone()),
-                        status_code: Some(StatusCode::BAD_REQUEST),
-                        message: format!(
-                            "Error parsing tool call arguments as JSON Value: {}",
-                            DisplayOrDebugGateway::new(e)
-                        ),
-                        provider_type: PROVIDER_TYPE.to_string(),
-                    })
-                })?;
+            // ...then convert the JSON Value to an AWS SDK Document
+            let input = serde_json::from_value(input).map_err(|e| {
+                Error::new(ErrorDetails::InferenceServer {
+                    raw_request: None,
+                    raw_response: None,
+                    message: format!(
+                        "Error converting tool call arguments to AWS SDK Document: {e}"
+                    ),
+                    provider_type: PROVIDER_TYPE.to_string(),
+                })
+            })?;
 
-                // ...then convert the JSON Value to an AWS SDK Document
-                let input = serde_json::from_value(input).map_err(|e| {
-                    Error::new(ErrorDetails::InferenceServer {
-                        raw_request: None,
-                        raw_response: None,
-                        message: format!(
-                            "Error converting tool call arguments to AWS SDK Document: {e}"
-                        ),
-                        provider_type: PROVIDER_TYPE.to_string(),
-                    })
-                })?;
-
-                let tool_use_block = ToolUseBlock::builder()
-                    .name(tool_call.name.clone())
-                    .input(input)
-                    .tool_use_id(tool_call.id.clone())
-                    .build()
-                    .map_err(|_| {
-                        Error::new(ErrorDetails::InferenceClient {
-                            raw_request: None,
-                            raw_response: None,
-                            status_code: Some(StatusCode::BAD_REQUEST),
-                            message: "Error serializing tool call block".to_string(),
-                            provider_type: PROVIDER_TYPE.to_string(),
-                        })
-                    })?;
-
-                Ok(Some(BedrockContentBlock::ToolUse(tool_use_block)))
-            }
-            ContentBlock::ToolResult(tool_result) => {
-                let tool_result_block = ToolResultBlock::builder()
-                    .tool_use_id(tool_result.id.clone())
-                    .content(ToolResultContentBlock::Text(tool_result.result.clone()))
-                    // NOTE: The AWS Bedrock SDK doesn't include `name` in the ToolResultBlock
-                    .build()
-                    .map_err(|_| {
-                        Error::new(ErrorDetails::InferenceClient {
-                            raw_request: None,
-                            raw_response: None,
-                            status_code: Some(StatusCode::BAD_REQUEST),
-                            message: "Error serializing tool result block".to_string(),
-                            provider_type: PROVIDER_TYPE.to_string(),
-                        })
-                    })?;
-
-                Ok(Some(BedrockContentBlock::ToolResult(tool_result_block)))
-            }
-            ContentBlock::File(file) => {
-                let FileWithPath {
-                    file,
-                    storage_path: _,
-                } = &**file;
-                let file_bytes = aws_smithy_types::base64::decode(file.data()?).map_err(|e| {
+            let tool_use_block = ToolUseBlock::builder()
+                .name(tool_call.name.clone())
+                .input(input)
+                .tool_use_id(tool_call.id.clone())
+                .build()
+                .map_err(|_| {
                     Error::new(ErrorDetails::InferenceClient {
                         raw_request: None,
                         raw_response: None,
                         status_code: Some(StatusCode::BAD_REQUEST),
-                        message: format!("File was not valid base64: {e:?}"),
+                        message: "Error serializing tool call block".to_string(),
                         provider_type: PROVIDER_TYPE.to_string(),
                     })
                 })?;
-                if file.mime_type.type_() == mime::IMAGE {
-                    let image_block = ImageBlock::builder()
-                        .format(ImageFormat::from(file.mime_type.subtype()))
-                        .source(ImageSource::Bytes(file_bytes.into()))
-                        .build()
-                        .map_err(|e| {
-                            Error::new(ErrorDetails::InferenceClient {
-                                raw_request: None,
-                                raw_response: None,
-                                status_code: Some(StatusCode::BAD_REQUEST),
-                                message: format!("Error serializing image block: {e:?}"),
-                                provider_type: PROVIDER_TYPE.to_string(),
-                            })
-                        })?;
-                    Ok(Some(BedrockContentBlock::Image(image_block)))
-                } else {
-                    // Best-effort attempt to produce an AWS DocumentFormat, as their API doesn't support mime types
-                    let suffix = mime_type_to_ext(&file.mime_type)?.ok_or_else(|| {
-                        Error::new(ErrorDetails::InvalidMessage {
-                            message: format!("Mime type {} has no filetype suffix", file.mime_type),
-                        })
-                    })?;
-                    let document_format = DocumentFormat::from(suffix);
-                    let document = DocumentBlock::builder()
-                        .format(document_format)
-                        // TODO: Should we allow the user to specify the file name?
-                        .name("input")
-                        .source(DocumentSource::Bytes(file_bytes.into()))
-                        .build()
-                        .map_err(|e| {
-                            Error::new(ErrorDetails::InferenceClient {
-                                raw_request: None,
-                                raw_response: None,
-                                status_code: Some(StatusCode::BAD_REQUEST),
-                                message: format!("Error serializing document block: {e:?}"),
-                                provider_type: PROVIDER_TYPE.to_string(),
-                            })
-                        })?;
-                    Ok(Some(BedrockContentBlock::Document(document)))
-                }
-            }
-            ContentBlock::Thought(thought) => {
-                if let Some(text) = &thought.text {
-                    let mut builder = ReasoningTextBlock::builder().text(text);
-                    if let Some(signature) = &thought.signature {
-                        builder = builder.signature(signature);
-                    }
-                    let block = builder.build().map_err(|e| {
-                        Error::new(ErrorDetails::InferenceClient {
-                            raw_request: None,
-                            raw_response: None,
-                            status_code: Some(StatusCode::BAD_REQUEST),
-                            message: format!("Error serializing reasoning text block: {e:?}"),
-                            provider_type: PROVIDER_TYPE.to_string(),
-                        })
-                    })?;
-                    Ok(Some(BedrockContentBlock::ReasoningContent(
-                        ReasoningContentBlock::ReasoningText(block),
-                    )))
-                } else if thought.signature.is_some() {
-                    tracing::warn!("The TensorZero Gateway doesn't support redacted thinking for AWS Bedrock yet, as none of the models available at the time of implementation supported this content block correctly. If you're seeing this warning, this means that something must have changed, so please reach out to our team and we'll quickly collaborate on a solution. For now, the gateway will discard such content blocks.");
-                    Ok(None)
-                } else {
-                    // We have a thought block with no text or signature, so just ignore it
-                    tracing::warn!("The gateway received a reasoning content block with neither text nor signature. This is unsupported, so we'll drop it.");
-                    Ok(None)
-                }
-            }
-            ContentBlock::Unknown {
-                data: _,
-                model_provider_name: _,
-            } => Err(Error::new(ErrorDetails::UnsupportedContentBlockType {
-                content_block_type: "unknown".to_string(),
-                provider_type: PROVIDER_TYPE.to_string(),
-            })),
+
+            Ok(Some(BedrockContentBlock::ToolUse(tool_use_block)))
         }
+        ContentBlock::ToolResult(tool_result) => {
+            let tool_result_block = ToolResultBlock::builder()
+                .tool_use_id(tool_result.id.clone())
+                .content(ToolResultContentBlock::Text(tool_result.result.clone()))
+                // NOTE: The AWS Bedrock SDK doesn't include `name` in the ToolResultBlock
+                .build()
+                .map_err(|_| {
+                    Error::new(ErrorDetails::InferenceClient {
+                        raw_request: None,
+                        raw_response: None,
+                        status_code: Some(StatusCode::BAD_REQUEST),
+                        message: "Error serializing tool result block".to_string(),
+                        provider_type: PROVIDER_TYPE.to_string(),
+                    })
+                })?;
+
+            Ok(Some(BedrockContentBlock::ToolResult(tool_result_block)))
+        }
+        ContentBlock::File(file) => {
+            let resolved_file = file.resolve().await?;
+            let FileWithPath {
+                file,
+                storage_path: _,
+            } = &*resolved_file;
+            let file_bytes = aws_smithy_types::base64::decode(file.data()?).map_err(|e| {
+                Error::new(ErrorDetails::InferenceClient {
+                    raw_request: None,
+                    raw_response: None,
+                    status_code: Some(StatusCode::BAD_REQUEST),
+                    message: format!("File was not valid base64: {e:?}"),
+                    provider_type: PROVIDER_TYPE.to_string(),
+                })
+            })?;
+            if file.mime_type.type_() == mime::IMAGE {
+                let image_block = ImageBlock::builder()
+                    .format(ImageFormat::from(file.mime_type.subtype()))
+                    .source(ImageSource::Bytes(file_bytes.into()))
+                    .build()
+                    .map_err(|e| {
+                        Error::new(ErrorDetails::InferenceClient {
+                            raw_request: None,
+                            raw_response: None,
+                            status_code: Some(StatusCode::BAD_REQUEST),
+                            message: format!("Error serializing image block: {e:?}"),
+                            provider_type: PROVIDER_TYPE.to_string(),
+                        })
+                    })?;
+                Ok(Some(BedrockContentBlock::Image(image_block)))
+            } else {
+                // Best-effort attempt to produce an AWS DocumentFormat, as their API doesn't support mime types
+                let suffix = mime_type_to_ext(&file.mime_type)?.ok_or_else(|| {
+                    Error::new(ErrorDetails::InvalidMessage {
+                        message: format!("Mime type {} has no filetype suffix", file.mime_type),
+                    })
+                })?;
+                let document_format = DocumentFormat::from(suffix);
+                let document = DocumentBlock::builder()
+                    .format(document_format)
+                    // TODO: Should we allow the user to specify the file name?
+                    .name("input")
+                    .source(DocumentSource::Bytes(file_bytes.into()))
+                    .build()
+                    .map_err(|e| {
+                        Error::new(ErrorDetails::InferenceClient {
+                            raw_request: None,
+                            raw_response: None,
+                            status_code: Some(StatusCode::BAD_REQUEST),
+                            message: format!("Error serializing document block: {e:?}"),
+                            provider_type: PROVIDER_TYPE.to_string(),
+                        })
+                    })?;
+                Ok(Some(BedrockContentBlock::Document(document)))
+            }
+        }
+        ContentBlock::Thought(thought) => {
+            if let Some(text) = &thought.text {
+                let mut builder = ReasoningTextBlock::builder().text(text);
+                if let Some(signature) = &thought.signature {
+                    builder = builder.signature(signature);
+                }
+                let block = builder.build().map_err(|e| {
+                    Error::new(ErrorDetails::InferenceClient {
+                        raw_request: None,
+                        raw_response: None,
+                        status_code: Some(StatusCode::BAD_REQUEST),
+                        message: format!("Error serializing reasoning text block: {e:?}"),
+                        provider_type: PROVIDER_TYPE.to_string(),
+                    })
+                })?;
+                Ok(Some(BedrockContentBlock::ReasoningContent(
+                    ReasoningContentBlock::ReasoningText(block),
+                )))
+            } else if thought.signature.is_some() {
+                tracing::warn!("The TensorZero Gateway doesn't support redacted thinking for AWS Bedrock yet, as none of the models available at the time of implementation supported this content block correctly. If you're seeing this warning, this means that something must have changed, so please reach out to our team and we'll quickly collaborate on a solution. For now, the gateway will discard such content blocks.");
+                Ok(None)
+            } else {
+                // We have a thought block with no text or signature, so just ignore it
+                tracing::warn!("The gateway received a reasoning content block with neither text nor signature. This is unsupported, so we'll drop it.");
+                Ok(None)
+            }
+        }
+        ContentBlock::Unknown {
+            data: _,
+            model_provider_name: _,
+        } => Err(Error::new(ErrorDetails::UnsupportedContentBlockType {
+            content_block_type: "unknown".to_string(),
+            provider_type: PROVIDER_TYPE.to_string(),
+        })),
     }
 }
 
@@ -809,34 +826,34 @@ fn bedrock_content_block_to_output(
     }
 }
 
-impl TryFrom<&RequestMessage> for Message {
-    type Error = Error;
-
-    fn try_from(inference_message: &RequestMessage) -> Result<Message, Error> {
-        let role: ConversationRole = inference_message.role.into();
-        let content: Vec<BedrockContentBlock> = inference_message
+// `Message` is a foreign type, so we cannot write an `impl` block on it
+async fn message_from_request_message(message: &RequestMessage) -> Result<Message, Error> {
+    let role: ConversationRole = message.role.into();
+    let content: Vec<BedrockContentBlock> = try_join_all(
+        message
             .content
             .iter()
-            .map(TryInto::try_into)
-            .collect::<Result<Vec<Option<BedrockContentBlock>>, _>>()?
-            .into_iter()
-            .flatten()
-            .collect();
-        let mut message_builder = Message::builder().role(role).set_content(Some(vec![]));
-        for block in content {
-            message_builder = message_builder.content(block);
-        }
-        let message = message_builder.build().map_err(|e| {
-            Error::new(ErrorDetails::InvalidMessage {
-                message: e.to_string(),
-            })
-        })?;
-
-        Ok(message)
+            .map(bedrock_content_block_from_content_block),
+    )
+    .await?
+    .into_iter()
+    .flatten()
+    .collect();
+    let mut message_builder = Message::builder().role(role).set_content(Some(vec![]));
+    for block in content {
+        message_builder = message_builder.content(block);
     }
+    let message = message_builder.build().map_err(|e| {
+        Error::new(ErrorDetails::InvalidMessage {
+            message: e.to_string(),
+        })
+    })?;
+
+    Ok(message)
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug)]
+#[cfg_attr(any(feature = "e2e_tests", test), derive(PartialEq))]
 struct ConverseOutputWithMetadata<'a> {
     output: ConverseOutput,
     latency: Latency,
