@@ -1,6 +1,6 @@
 use futures::StreamExt;
 use lazy_static::lazy_static;
-use reqwest_eventsource::{Event, EventSource};
+use reqwest_eventsource::Event;
 use secrecy::{ExposeSecret, SecretString};
 use serde::{Deserialize, Serialize};
 use std::borrow::Cow;
@@ -14,6 +14,7 @@ use super::helpers::{
 use crate::cache::ModelProviderRequest;
 use crate::endpoints::inference::InferenceCredentials;
 use crate::error::{DisplayOrDebugGateway, Error, ErrorDetails};
+use crate::http::{TensorZeroEventSource, TensorzeroHttpClient};
 use crate::inference::types::batch::{BatchRequestRow, PollBatchInferenceResponse};
 use crate::inference::types::{
     batch::StartBatchProviderInferenceResponse, Latency, ModelInferenceRequest,
@@ -32,10 +33,10 @@ use crate::tool::ToolCallChunk;
 
 use super::openai::{
     convert_stream_error, get_chat_url, handle_openai_error, prepare_openai_tools,
-    tensorzero_to_openai_messages, tensorzero_to_openai_system_message,
+    prepare_system_or_developer_message, tensorzero_to_openai_messages,
     OpenAIAssistantRequestMessage, OpenAIContentBlock, OpenAIFinishReason, OpenAIRequestMessage,
     OpenAIResponseToolCall, OpenAISystemRequestMessage, OpenAITool, OpenAIToolChoice, OpenAIUsage,
-    OpenAIUserRequestMessage, StreamOptions,
+    OpenAIUserRequestMessage, StreamOptions, SystemOrDeveloper,
 };
 
 lazy_static! {
@@ -86,12 +87,14 @@ impl DeepSeekCredentials {
                 dynamic_api_keys.get(key_name).ok_or_else(|| {
                     ErrorDetails::ApiKeyMissing {
                         provider_name: PROVIDER_NAME.to_string(),
+                        message: format!("Dynamic api key `{key_name}` is missing"),
                     }
                     .into()
                 })
             }
             DeepSeekCredentials::None => Err(ErrorDetails::ApiKeyMissing {
                 provider_name: PROVIDER_NAME.to_string(),
+                message: "No credentials are set".to_string(),
             }
             .into()),
         }
@@ -112,7 +115,7 @@ impl DeepSeekProvider {
         model_name: String,
         api_key_location: Option<CredentialLocation>,
     ) -> Result<Self, Error> {
-        let credential_location = api_key_location.unwrap_or(default_api_key_location());
+        let credential_location = api_key_location.unwrap_or_else(default_api_key_location);
         let generic_credentials = Credential::try_from((credential_location, PROVIDER_TYPE))?;
         let provider_credentials = DeepSeekCredentials::try_from(generic_credentials)?;
 
@@ -134,13 +137,16 @@ impl InferenceProvider for DeepSeekProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         dynamic_api_keys: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<ProviderInferenceResponse, Error> {
-        let request_body = serde_json::to_value(DeepSeekRequest::new(&self.model_name, request)?)
-            .map_err(|e| {
+        let request_body = serde_json::to_value(
+            DeepSeekRequest::new(&self.model_name, request).await?,
+        )
+        .map_err(|e| {
             Error::new(ErrorDetails::Serialization {
                 message: format!(
                     "Error serializing DeepSeek request: {}",
@@ -232,13 +238,16 @@ impl InferenceProvider for DeepSeekProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         dynamic_api_keys: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<(PeekableProviderInferenceResponseStream, String), Error> {
-        let request_body = serde_json::to_value(DeepSeekRequest::new(&self.model_name, request)?)
-            .map_err(|e| {
+        let request_body = serde_json::to_value(
+            DeepSeekRequest::new(&self.model_name, request).await?,
+        )
+        .map_err(|e| {
             Error::new(ErrorDetails::Serialization {
                 message: format!(
                     "Error serializing DeepSeek request: {}",
@@ -271,7 +280,7 @@ impl InferenceProvider for DeepSeekProvider {
     async fn start_batch_inference<'a>(
         &'a self,
         _requests: &'a [ModelInferenceRequest<'_>],
-        _client: &'a reqwest::Client,
+        _client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<StartBatchProviderInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -283,7 +292,7 @@ impl InferenceProvider for DeepSeekProvider {
     async fn poll_batch_inference<'a>(
         &'a self,
         _batch_request: &'a BatchRequestRow<'a>,
-        _http_client: &'a reqwest::Client,
+        _http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<PollBatchInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -344,7 +353,7 @@ struct DeepSeekRequest<'a> {
 }
 
 impl<'a> DeepSeekRequest<'a> {
-    pub fn new(
+    pub async fn new(
         model: &'a str,
         request: &'a ModelInferenceRequest<'_>,
     ) -> Result<DeepSeekRequest<'a>, Error> {
@@ -376,7 +385,7 @@ impl<'a> DeepSeekRequest<'a> {
         // NOTE: as mentioned by the DeepSeek team here: https://github.com/deepseek-ai/DeepSeek-R1?tab=readme-ov-file#usage-recommendations
         // the R1 series of models does not perform well with the system prompt. As we move towards first-class support for reasoning models we should check
         // if a model is an R1 model and if so, remove the system prompt from the request and instead put it in the first user message.
-        let messages = prepare_deepseek_messages(request, model)?;
+        let messages = prepare_deepseek_messages(request, model).await?;
 
         let (tools, tool_choice, _) = prepare_openai_tools(request);
 
@@ -400,7 +409,7 @@ impl<'a> DeepSeekRequest<'a> {
 }
 
 fn stream_deepseek(
-    mut event_source: EventSource,
+    mut event_source: TensorZeroEventSource,
     start_time: Instant,
 ) -> ProviderInferenceResponseStreamInner {
     let mut tool_call_ids = Vec::new();
@@ -584,13 +593,13 @@ struct DeepSeekResponse {
     usage: OpenAIUsage,
 }
 
-pub(super) fn prepare_deepseek_messages<'a>(
+pub(super) async fn prepare_deepseek_messages<'a>(
     request: &'a ModelInferenceRequest<'_>,
     model_name: &'a str,
 ) -> Result<Vec<OpenAIRequestMessage<'a>>, Error> {
     let mut messages = Vec::with_capacity(request.messages.len());
     for message in &request.messages {
-        messages.extend(tensorzero_to_openai_messages(message, PROVIDER_TYPE)?);
+        messages.extend(tensorzero_to_openai_messages(message, PROVIDER_TYPE).await?);
     }
     // If this is an R1 model, prepend the system message as the first user message instead of using it as a system message
     if model_name.to_lowercase().contains("reasoner") {
@@ -604,8 +613,8 @@ pub(super) fn prepare_deepseek_messages<'a>(
                 }),
             );
         }
-    } else if let Some(system_msg) = tensorzero_to_openai_system_message(
-        request.system.as_deref(),
+    } else if let Some(system_msg) = prepare_system_or_developer_message(
+        request.system.as_deref().map(SystemOrDeveloper::System),
         Some(&request.json_mode),
         &messages,
     ) {
@@ -767,8 +776,8 @@ mod tests {
     };
     use crate::providers::test_helpers::{WEATHER_TOOL, WEATHER_TOOL_CONFIG};
 
-    #[test]
-    fn test_deepseek_request_new() {
+    #[tokio::test]
+    async fn test_deepseek_request_new() {
         let request_with_tools = ModelInferenceRequest {
             inference_id: Uuid::now_v7(),
             messages: vec![RequestMessage {
@@ -792,6 +801,7 @@ mod tests {
         };
 
         let deepseek_request = DeepSeekRequest::new("deepseek-chat", &request_with_tools)
+            .await
             .expect("failed to create Deepseek Request during test");
 
         assert_eq!(deepseek_request.messages.len(), 1);
@@ -838,6 +848,7 @@ mod tests {
         };
 
         let deepseek_request = DeepSeekRequest::new("deepseek-chat", &request_with_tools)
+            .await
             .expect("failed to create Deepseek Request");
 
         assert_eq!(deepseek_request.messages.len(), 2);
@@ -870,7 +881,7 @@ mod tests {
             ..request_with_tools
         };
 
-        let deepseek_request = DeepSeekRequest::new("deepseek-chat", &request_with_tools);
+        let deepseek_request = DeepSeekRequest::new("deepseek-chat", &request_with_tools).await;
         let deepseek_request = deepseek_request.unwrap();
         // We should downgrade the strict JSON mode to normal JSON mode for deepseek
         assert_eq!(
@@ -879,16 +890,16 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_deepseek_api_base() {
+    #[tokio::test]
+    async fn test_deepseek_api_base() {
         assert_eq!(
             DEEPSEEK_DEFAULT_BASE_URL.as_str(),
             "https://api.deepseek.com/v1"
         );
     }
 
-    #[test]
-    fn test_credential_to_deepseek_credentials() {
+    #[tokio::test]
+    async fn test_credential_to_deepseek_credentials() {
         // Test Static credential
         let generic = Credential::Static(SecretString::from("test_key"));
         let creds: DeepSeekCredentials = DeepSeekCredentials::try_from(generic).unwrap();
@@ -909,12 +920,12 @@ mod tests {
         let result = DeepSeekCredentials::try_from(generic);
         assert!(result.is_err());
         assert!(matches!(
-            result.unwrap_err().get_owned_details(),
+            result.unwrap_err().get_details(),
             ErrorDetails::Config { message } if message.contains("Invalid api_key_location")
         ));
     }
-    #[test]
-    fn test_deepseek_response_with_metadata_try_into() {
+    #[tokio::test]
+    async fn test_deepseek_response_with_metadata_try_into() {
         let valid_response = DeepSeekResponse {
             choices: vec![DeepSeekResponseChoice {
                 index: 0,
@@ -959,7 +970,9 @@ mod tests {
                 response_time: Duration::from_secs(0),
             },
             raw_request: serde_json::to_string(
-                &DeepSeekRequest::new("deepseek-chat", &generic_request).unwrap(),
+                &DeepSeekRequest::new("deepseek-chat", &generic_request)
+                    .await
+                    .unwrap(),
             )
             .unwrap(),
             generic_request: &generic_request,
@@ -993,8 +1006,8 @@ mod tests {
         assert_eq!(inference_response.finish_reason, Some(FinishReason::Stop));
     }
 
-    #[test]
-    fn test_prepare_deepseek_messages() {
+    #[tokio::test]
+    async fn test_prepare_deepseek_messages() {
         // Test case 1: Regular model with system message
         let request = ModelInferenceRequest {
             inference_id: Uuid::now_v7(),
@@ -1018,13 +1031,17 @@ mod tests {
             ..Default::default()
         };
 
-        let messages = prepare_deepseek_messages(&request, "deepseek-chat").unwrap();
+        let messages = prepare_deepseek_messages(&request, "deepseek-chat")
+            .await
+            .unwrap();
         assert_eq!(messages.len(), 2);
         assert!(matches!(messages[0], OpenAIRequestMessage::System(_)));
         assert!(matches!(messages[1], OpenAIRequestMessage::User(_)));
 
         // Test case 2: Reasoner model with system message
-        let messages = prepare_deepseek_messages(&request, "deepseek-reasoner").unwrap();
+        let messages = prepare_deepseek_messages(&request, "deepseek-reasoner")
+            .await
+            .unwrap();
         assert_eq!(messages.len(), 1);
         match &messages[0] {
             OpenAIRequestMessage::User(user_msg) => {
@@ -1066,7 +1083,9 @@ mod tests {
             ..Default::default()
         };
 
-        let messages = prepare_deepseek_messages(&request_no_system, "deepseek-chat").unwrap();
+        let messages = prepare_deepseek_messages(&request_no_system, "deepseek-chat")
+            .await
+            .unwrap();
         assert_eq!(messages.len(), 1);
         assert!(matches!(messages[0], OpenAIRequestMessage::User(_)));
 
@@ -1103,7 +1122,9 @@ mod tests {
             ..Default::default()
         };
 
-        let messages = prepare_deepseek_messages(&request_multiple, "deepseek-chat").unwrap();
+        let messages = prepare_deepseek_messages(&request_multiple, "deepseek-chat")
+            .await
+            .unwrap();
         assert_eq!(messages.len(), 4);
         assert!(matches!(messages[0], OpenAIRequestMessage::System(_)));
         assert!(matches!(messages[1], OpenAIRequestMessage::User(_)));
@@ -1140,8 +1161,8 @@ mod tests {
         })
     }
 
-    #[test]
-    fn test_coalesce_consecutive_messages() {
+    #[tokio::test]
+    async fn test_coalesce_consecutive_messages() {
         // Create dummy tool calls to test assistant message merging.
         let tool_call1 = OpenAIRequestToolCall {
             id: "tc1".into(),
