@@ -17,8 +17,10 @@ use crate::embeddings::{
     Embedding, EmbeddingProvider, EmbeddingProviderRequestInfo, EmbeddingProviderResponse,
     EmbeddingRequest,
 };
+
 use crate::endpoints::inference::InferenceCredentials;
 use crate::error::{Error, ErrorDetails};
+use crate::http::TensorzeroHttpClient;
 use crate::inference::types::batch::PollBatchInferenceResponse;
 use crate::inference::types::batch::{BatchRequestRow, BatchStatus};
 use crate::inference::types::{
@@ -30,6 +32,7 @@ use crate::inference::types::{ContentBlock, FinishReason, ProviderInferenceRespo
 use crate::inference::types::{Text, TextChunk, Thought, ThoughtChunk};
 use crate::model::{CredentialLocation, ModelProvider};
 use crate::providers::helpers::inject_extra_request_data;
+use crate::rate_limiting::ActiveRateLimitKey;
 use crate::tool::{ToolCall, ToolCallChunk};
 
 const PROVIDER_NAME: &str = "Dummy";
@@ -49,7 +52,7 @@ impl DummyProvider {
         model_name: String,
         api_key_location: Option<CredentialLocation>,
     ) -> Result<Self, Error> {
-        let api_key_location = api_key_location.unwrap_or(default_api_key_location());
+        let api_key_location = api_key_location.unwrap_or_else(default_api_key_location);
         match api_key_location {
             CredentialLocation::Dynamic(key_name) => Ok(DummyProvider {
                 model_name,
@@ -165,6 +168,7 @@ impl DummyCredentials {
                 Some(dynamic_api_keys.get(key_name).ok_or_else(|| {
                     ErrorDetails::ApiKeyMissing {
                         provider_name: PROVIDER_NAME.to_string(),
+                        message: format!("Dynamic api key `{key_name}` is missing"),
                     }
                     .into()
                 }))
@@ -244,14 +248,17 @@ impl InferenceProvider for DummyProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        _http_client: &'a reqwest::Client,
+        _http_client: &'a TensorzeroHttpClient,
         dynamic_api_keys: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<ProviderInferenceResponse, Error> {
         if self.model_name == "slow" {
             tokio::time::sleep(Duration::from_secs(5)).await;
         }
+        // Just so they don't seem like 0ms inferences
+        tokio::time::sleep(Duration::from_millis(1)).await;
 
         // Check for flaky models
         if self.model_name.starts_with("flaky_") {
@@ -264,6 +271,13 @@ impl InferenceProvider for DummyProvider {
 
             // Fail on even-numbered calls
             if *counter % 2 == 0 {
+                if self.model_name.contains("rate_limit") {
+                    return Err(ErrorDetails::RateLimitExceeded {
+                        key: ActiveRateLimitKey(String::from("key")),
+                        tickets_remaining: 0,
+                    }
+                    .into());
+                }
                 return Err(ErrorDetails::InferenceClient {
                     raw_request: Some("raw request".to_string()),
                     raw_response: None,
@@ -477,7 +491,8 @@ impl InferenceProvider for DummyProvider {
                     .collect();
                 let mut found_pdf = false;
                 for file in &files {
-                    if file.file.mime_type == mime::APPLICATION_PDF {
+                    let resolved_file = file.resolve().await?;
+                    if resolved_file.file.mime_type == mime::APPLICATION_PDF {
                         found_pdf = true;
                     }
                 }
@@ -561,14 +576,17 @@ impl InferenceProvider for DummyProvider {
             request: _,
             provider_name: _,
             model_name: _,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        _http_client: &'a reqwest::Client,
+        _http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
         _model_provider: &'a ModelProvider,
     ) -> Result<(PeekableProviderInferenceResponseStream, String), Error> {
         if self.model_name == "slow" {
             tokio::time::sleep(Duration::from_secs(5)).await;
         }
+        // Just so they don't seem like 0ms inferences
+        tokio::time::sleep(Duration::from_millis(1)).await;
         // Check for flaky models
         if self.model_name.starts_with("flaky_") {
             #[expect(clippy::expect_used)]
@@ -718,7 +736,7 @@ impl InferenceProvider for DummyProvider {
     async fn start_batch_inference<'a>(
         &'a self,
         requests: &'a [ModelInferenceRequest<'_>],
-        _client: &'a reqwest::Client,
+        _client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<StartBatchProviderInferenceResponse, Error> {
         let file_id = Uuid::now_v7();
@@ -739,7 +757,7 @@ impl InferenceProvider for DummyProvider {
     async fn poll_batch_inference<'a>(
         &'a self,
         _batch_request: &'a BatchRequestRow<'a>,
-        _http_client: &'a reqwest::Client,
+        _http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<PollBatchInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -756,7 +774,7 @@ impl EmbeddingProvider for DummyProvider {
     async fn embed(
         &self,
         request: &EmbeddingRequest,
-        _http_client: &reqwest::Client,
+        _http_client: &TensorzeroHttpClient,
         _dynamic_api_keys: &InferenceCredentials,
         _model_provider_data: &EmbeddingProviderRequestInfo,
     ) -> Result<EmbeddingProviderResponse, Error> {
