@@ -1,9 +1,12 @@
 use std::{borrow::Cow, sync::OnceLock};
 
-use crate::{providers::helpers_thinking_block::THINK_CHUNK_ID, tool::Tool};
+use crate::http::TensorzeroHttpClient;
+use crate::{
+    http::TensorZeroEventSource, providers::helpers_thinking_block::THINK_CHUNK_ID, tool::Tool,
+};
 use futures::StreamExt;
 use lazy_static::lazy_static;
-use reqwest_eventsource::{Event, EventSource};
+use reqwest_eventsource::Event;
 use secrecy::{ExposeSecret, SecretString};
 use serde::{Deserialize, Serialize};
 use serde_json::Value;
@@ -132,12 +135,14 @@ impl FireworksCredentials {
                 dynamic_api_keys.get(key_name).ok_or_else(|| {
                     ErrorDetails::ApiKeyMissing {
                         provider_name: PROVIDER_NAME.to_string(),
+                        message: format!("Dynamic api key `{key_name}` is missing"),
                     }
                     .into()
                 })
             }
             &FireworksCredentials::None => Err(ErrorDetails::ApiKeyMissing {
                 provider_name: PROVIDER_NAME.to_string(),
+                message: "No credentials are set".to_string(),
             }
             .into()),
         }
@@ -160,20 +165,23 @@ impl InferenceProvider for FireworksProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         api_key: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<ProviderInferenceResponse, Error> {
-        let request_body = serde_json::to_value(FireworksRequest::new(&self.model_name, request)?)
-            .map_err(|e| {
-                Error::new(ErrorDetails::Serialization {
-                    message: format!(
-                        "Error serializing Fireworks request: {}",
-                        DisplayOrDebugGateway::new(e)
-                    ),
-                })
-            })?;
+        let request_body = serde_json::to_value(
+            FireworksRequest::new(&self.model_name, request).await?,
+        )
+        .map_err(|e| {
+            Error::new(ErrorDetails::Serialization {
+                message: format!(
+                    "Error serializing Fireworks request: {}",
+                    DisplayOrDebugGateway::new(e)
+                ),
+            })
+        })?;
         let request_url = get_chat_url(&FIREWORKS_API_INFERENCE_BASE)?;
         let start_time = Instant::now();
         let api_key = self.credentials.get_api_key(api_key)?;
@@ -250,20 +258,23 @@ impl InferenceProvider for FireworksProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         api_key: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<(PeekableProviderInferenceResponseStream, String), Error> {
-        let request_body = serde_json::to_value(FireworksRequest::new(&self.model_name, request)?)
-            .map_err(|e| {
-                Error::new(ErrorDetails::Serialization {
-                    message: format!(
-                        "Error serializing Fireworks request: {}",
-                        DisplayOrDebugGateway::new(e)
-                    ),
-                })
-            })?;
+        let request_body = serde_json::to_value(
+            FireworksRequest::new(&self.model_name, request).await?,
+        )
+        .map_err(|e| {
+            Error::new(ErrorDetails::Serialization {
+                message: format!(
+                    "Error serializing Fireworks request: {}",
+                    DisplayOrDebugGateway::new(e)
+                ),
+            })
+        })?;
         let request_url = get_chat_url(&FIREWORKS_API_INFERENCE_BASE)?;
         let api_key = self.credentials.get_api_key(api_key)?;
         let start_time = Instant::now();
@@ -288,7 +299,7 @@ impl InferenceProvider for FireworksProvider {
     async fn start_batch_inference<'a>(
         &'a self,
         _requests: &'a [ModelInferenceRequest<'_>],
-        _client: &'a reqwest::Client,
+        _client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<StartBatchProviderInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -300,7 +311,7 @@ impl InferenceProvider for FireworksProvider {
     async fn poll_batch_inference<'a>(
         &'a self,
         _batch_request: &'a BatchRequestRow<'a>,
-        _http_client: &'a reqwest::Client,
+        _http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<PollBatchInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -355,7 +366,7 @@ struct FireworksRequest<'a> {
 }
 
 impl<'a> FireworksRequest<'a> {
-    pub fn new(
+    pub async fn new(
         model: &'a str,
         request: &'a ModelInferenceRequest<'_>,
     ) -> Result<FireworksRequest<'a>, Error> {
@@ -369,7 +380,8 @@ impl<'a> FireworksRequest<'a> {
             }
             ModelInferenceRequestJsonMode::Off => None,
         };
-        let messages = prepare_fireworks_messages(request.system.as_deref(), &request.messages)?;
+        let messages =
+            prepare_fireworks_messages(request.system.as_deref(), &request.messages).await?;
         let (tools, tool_choice, _) = prepare_openai_tools(request);
         let tools = tools.map(|t| t.into_iter().map(OpenAITool::into).collect());
 
@@ -390,13 +402,13 @@ impl<'a> FireworksRequest<'a> {
     }
 }
 
-pub fn prepare_fireworks_messages<'a>(
+pub async fn prepare_fireworks_messages<'a>(
     system: Option<&'a str>,
     messages: &'a [RequestMessage],
 ) -> Result<Vec<OpenAIRequestMessage<'a>>, Error> {
     let mut output_messages = Vec::with_capacity(messages.len());
     for message in messages {
-        output_messages.extend(tensorzero_to_openai_messages(message, PROVIDER_TYPE)?);
+        output_messages.extend(tensorzero_to_openai_messages(message, PROVIDER_TYPE).await?);
     }
     if let Some(system_msg) = tensorzero_to_fireworks_system_message(system) {
         output_messages.insert(0, system_msg);
@@ -555,7 +567,7 @@ struct FireworksChatChunk {
 /// Streams the Fireworks response events and converts them into ProviderInferenceResponseChunks
 /// This function handles parsing and processing of thinking blocks with proper state tracking
 fn stream_fireworks(
-    mut event_source: EventSource,
+    mut event_source: TensorZeroEventSource,
     start_time: Instant,
     parse_think_blocks: bool,
 ) -> ProviderInferenceResponseStreamInner {
@@ -814,8 +826,8 @@ mod tests {
     use crate::providers::openai::{SpecificToolChoice, SpecificToolFunction};
     use crate::providers::test_helpers::{WEATHER_TOOL, WEATHER_TOOL_CONFIG};
 
-    #[test]
-    fn test_fireworks_response_with_thinking_blocks() {
+    #[tokio::test]
+    async fn test_fireworks_response_with_thinking_blocks() {
         let test_response_with_thinking = "Hello <think>This is reasoning</think> world";
         let generic_request = ModelInferenceRequest {
             inference_id: Uuid::now_v7(),
@@ -865,7 +877,9 @@ mod tests {
                 response_time: Duration::from_secs(0),
             },
             raw_request: serde_json::to_string(
-                &FireworksRequest::new("test-model", &generic_request).unwrap(),
+                &FireworksRequest::new("test-model", &generic_request)
+                    .await
+                    .unwrap(),
             )
             .unwrap(),
             generic_request: &generic_request,
@@ -903,7 +917,9 @@ mod tests {
                 response_time: Duration::from_secs(0),
             },
             raw_request: serde_json::to_string(
-                &FireworksRequest::new("test-model", &generic_request).unwrap(),
+                &FireworksRequest::new("test-model", &generic_request)
+                    .await
+                    .unwrap(),
             )
             .unwrap(),
             generic_request: &generic_request,
@@ -925,8 +941,8 @@ mod tests {
         }
     }
 
-    #[test]
-    fn test_fireworks_request_new() {
+    #[tokio::test]
+    async fn test_fireworks_request_new() {
         let request_with_tools = ModelInferenceRequest {
             inference_id: Uuid::now_v7(),
             messages: vec![RequestMessage {
@@ -951,6 +967,7 @@ mod tests {
 
         let fireworks_request =
             FireworksRequest::new("accounts/fireworks/models/llama-v3-8b", &request_with_tools)
+                .await
                 .unwrap();
 
         assert_eq!(
@@ -986,16 +1003,16 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_fireworks_api_base() {
+    #[tokio::test]
+    async fn test_fireworks_api_base() {
         assert_eq!(
             FIREWORKS_API_INFERENCE_BASE.as_str(),
             "https://api.fireworks.ai/inference/v1/"
         );
     }
 
-    #[test]
-    fn test_credential_to_fireworks_credentials() {
+    #[tokio::test]
+    async fn test_credential_to_fireworks_credentials() {
         // Test Static credential
         let generic = Credential::Static(SecretString::from("test_key"));
         let creds = FireworksCredentials::try_from(generic).unwrap();
@@ -1016,13 +1033,13 @@ mod tests {
         let result = FireworksCredentials::try_from(generic);
         assert!(result.is_err());
         assert!(matches!(
-            result.unwrap_err().get_owned_details(),
+            result.unwrap_err().get_details(),
             ErrorDetails::Config { message } if message.contains("Invalid api_key_location")
         ));
     }
 
-    #[test]
-    fn test_fireworks_response_with_metadata_try_into() {
+    #[tokio::test]
+    async fn test_fireworks_response_with_metadata_try_into() {
         let valid_response = FireworksResponse {
             choices: vec![FireworksResponseChoice {
                 index: 0,
@@ -1067,7 +1084,9 @@ mod tests {
                 response_time: Duration::from_secs(0),
             },
             raw_request: serde_json::to_string(
-                &FireworksRequest::new("test-model", &generic_request).unwrap(),
+                &FireworksRequest::new("test-model", &generic_request)
+                    .await
+                    .unwrap(),
             )
             .unwrap(),
             generic_request: &generic_request,
@@ -1092,8 +1111,8 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_fireworks_to_tensorzero_chunk() {
+    #[tokio::test]
+    async fn test_fireworks_to_tensorzero_chunk() {
         let chunk = FireworksChatChunk {
             choices: vec![FireworksChatChunkChoice {
                 delta: FireworksDelta {
@@ -1194,8 +1213,8 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_fireworks_to_tensorzero_chunk_thinking() {
+    #[tokio::test]
+    async fn test_fireworks_to_tensorzero_chunk_thinking() {
         // Test that the streaming function correctly handles thinking blocks
         let chunk = FireworksChatChunk {
             choices: vec![FireworksChatChunkChoice {
@@ -1323,8 +1342,8 @@ mod tests {
         }
     }
 
-    #[test]
-    fn test_fireworks_to_tensorzero_chunk_without_think_parsing() {
+    #[tokio::test]
+    async fn test_fireworks_to_tensorzero_chunk_without_think_parsing() {
         let chunk = FireworksChatChunk {
             choices: vec![FireworksChatChunkChoice {
                 delta: FireworksDelta {
@@ -1356,8 +1375,8 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_fireworks_stream_tool_call_handling() {
+    #[tokio::test]
+    async fn test_fireworks_stream_tool_call_handling() {
         // Test new tool call with ID and name
         let chunk = FireworksChatChunk {
             choices: vec![FireworksChatChunkChoice {
