@@ -1,3 +1,5 @@
+use crate::http::TensorzeroHttpClient;
+use async_trait::async_trait;
 /// TGI integration for TensorZero
 ///
 /// Here, we list known limitations of TGI in our experience
@@ -26,6 +28,7 @@ use url::Url;
 use super::openai::{
     convert_stream_error, get_chat_url, prepare_openai_messages, prepare_openai_tools,
     OpenAIRequestMessage, OpenAITool, OpenAIToolChoice, OpenAIToolType, StreamOptions,
+    SystemOrDeveloper,
 };
 use crate::cache::ModelProviderRequest;
 use crate::endpoints::inference::InferenceCredentials;
@@ -121,6 +124,7 @@ impl TGICredentials {
                 Some(dynamic_api_keys.get(key_name).ok_or_else(|| {
                     ErrorDetails::ApiKeyMissing {
                         provider_name: PROVIDER_NAME.to_string(),
+                        message: format!("Dynamic api key `{key_name}` is missing"),
                     }
                     .into()
                 }))
@@ -131,22 +135,24 @@ impl TGICredentials {
     }
 }
 
+#[async_trait]
 impl WrappedProvider for TGIProvider {
     fn thought_block_provider_type_suffix(&self) -> Cow<'static, str> {
         Cow::Borrowed("tgi")
     }
 
-    fn make_body<'a>(
+    async fn make_body<'a>(
         &'a self,
         ModelProviderRequest {
             request,
             provider_name: _,
             model_name: _,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
     ) -> Result<serde_json::Value, Error> {
         // TGI doesn't care about the `model_name` field, so we can hardcode it to "tgi"
 
-        serde_json::to_value(TGIRequest::new(PROVIDER_TYPE, request)?).map_err(|e| {
+        serde_json::to_value(TGIRequest::new(PROVIDER_TYPE, request).await?).map_err(|e| {
             Error::new(ErrorDetails::Serialization {
                 message: format!(
                     "Error serializing TGI request: {}",
@@ -200,11 +206,11 @@ impl InferenceProvider for TGIProvider {
     async fn infer<'a>(
         &'a self,
         model_provider_request: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         dynamic_api_keys: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<ProviderInferenceResponse, Error> {
-        let request_body = self.make_body(model_provider_request)?;
+        let request_body = self.make_body(model_provider_request).await?;
         let request_url = get_chat_url(&self.api_base)?;
         let api_key = self.credentials.get_api_key(dynamic_api_keys)?;
         let start_time = Instant::now();
@@ -272,13 +278,14 @@ impl InferenceProvider for TGIProvider {
             request,
             provider_name: _,
             model_name,
+            otlp_config: _,
         }: ModelProviderRequest<'a>,
-        http_client: &'a reqwest::Client,
+        http_client: &'a TensorzeroHttpClient,
         dynamic_api_keys: &'a InferenceCredentials,
         model_provider: &'a ModelProvider,
     ) -> Result<(PeekableProviderInferenceResponseStream, String), Error> {
-        let request_body =
-            serde_json::to_value(TGIRequest::new(PROVIDER_NAME, request)?).map_err(|e| {
+        let request_body = serde_json::to_value(TGIRequest::new(PROVIDER_NAME, request).await?)
+            .map_err(|e| {
                 Error::new(ErrorDetails::Serialization {
                     message: format!(
                         "Error serializing TGI request: {}",
@@ -323,7 +330,7 @@ impl InferenceProvider for TGIProvider {
     async fn start_batch_inference<'a>(
         &'a self,
         _requests: &'a [ModelInferenceRequest<'_>],
-        _client: &'a reqwest::Client,
+        _client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<StartBatchProviderInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -335,7 +342,7 @@ impl InferenceProvider for TGIProvider {
     async fn poll_batch_inference<'a>(
         &'a self,
         _batch_request: &'a BatchRequestRow<'a>,
-        _http_client: &'a reqwest::Client,
+        _http_client: &'a TensorzeroHttpClient,
         _dynamic_api_keys: &'a InferenceCredentials,
     ) -> Result<PollBatchInferenceResponse, Error> {
         Err(ErrorDetails::UnsupportedModelProviderForBatchInference {
@@ -427,7 +434,7 @@ struct TGIRequest<'a> {
 }
 
 impl<'a> TGIRequest<'a> {
-    pub fn new(
+    pub async fn new(
         model: &'a str,
         request: &'a ModelInferenceRequest<'_>,
     ) -> Result<TGIRequest<'a>, Error> {
@@ -447,11 +454,12 @@ impl<'a> TGIRequest<'a> {
         };
 
         let messages = prepare_openai_messages(
-            request.system.as_deref(),
+            request.system.as_deref().map(SystemOrDeveloper::System),
             &request.messages,
             Some(&request.json_mode),
             PROVIDER_TYPE,
-        )?;
+        )
+        .await?;
 
         let (tools, tool_choice, parallel_tool_calls) = prepare_openai_tools(request);
 
@@ -474,19 +482,6 @@ impl<'a> TGIRequest<'a> {
     }
 }
 
-#[derive(Serialize, Debug, Clone, PartialEq, Deserialize)]
-struct OpenAIRequestToolCall<'a> {
-    id: &'a str,
-    r#type: OpenAIToolType,
-    function: OpenAIRequestFunctionCall<'a>,
-}
-
-#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
-struct OpenAIRequestFunctionCall<'a> {
-    name: &'a str,
-    arguments: &'a str,
-}
-
 struct TGIResponseWithMetadata<'a> {
     response: TGIResponse,
     latency: Latency,
@@ -779,8 +774,8 @@ mod tests {
 
     use super::*;
 
-    #[test]
-    fn test_tgi_request_new() {
+    #[tokio::test]
+    async fn test_tgi_request_new() {
         let model_name = PROVIDER_TYPE.to_string();
         let basic_request = ModelInferenceRequest {
             inference_id: Uuid::now_v7(),
@@ -809,7 +804,7 @@ mod tests {
             extra_body: Default::default(),
             ..Default::default()
         };
-        let tgi_request = TGIRequest::new(&model_name, &basic_request).unwrap();
+        let tgi_request = TGIRequest::new(&model_name, &basic_request).await.unwrap();
 
         assert_eq!(tgi_request.model, &model_name);
         assert_eq!(tgi_request.messages.len(), 2);
@@ -847,7 +842,9 @@ mod tests {
             ..Default::default()
         };
 
-        let tgi_request = TGIRequest::new(&model_name, &request_with_tools).unwrap();
+        let tgi_request = TGIRequest::new(&model_name, &request_with_tools)
+            .await
+            .unwrap();
 
         assert_eq!(tgi_request.model, &model_name);
         assert_eq!(tgi_request.messages.len(), 2);
@@ -895,7 +892,9 @@ mod tests {
             ..Default::default()
         };
 
-        let tgi_request = TGIRequest::new(&model_name, &request_with_tools).unwrap();
+        let tgi_request = TGIRequest::new(&model_name, &request_with_tools)
+            .await
+            .unwrap();
 
         assert_eq!(tgi_request.model, &model_name);
         assert_eq!(tgi_request.messages.len(), 1);
@@ -931,7 +930,9 @@ mod tests {
             ..Default::default()
         };
 
-        let tgi_request = TGIRequest::new(&model_name, &request_with_tools).unwrap();
+        let tgi_request = TGIRequest::new(&model_name, &request_with_tools)
+            .await
+            .unwrap();
 
         assert_eq!(tgi_request.model, &model_name);
         assert_eq!(tgi_request.messages.len(), 1);
@@ -971,8 +972,8 @@ mod tests {
         );
         assert_eq!(message.finish_reason, Some(FinishReason::Stop));
     }
-    #[test]
-    fn test_tgi_response_with_metadata_try_into() {
+    #[tokio::test]
+    async fn test_tgi_response_with_metadata_try_into() {
         let valid_response = TGIResponse {
             choices: vec![TGIResponseChoice {
                 index: 0,
@@ -1016,7 +1017,9 @@ mod tests {
                 response_time: Duration::from_secs(0),
             },
             raw_request: serde_json::to_value(
-                TGIRequest::new("test-model", &generic_request).unwrap(),
+                TGIRequest::new("test-model", &generic_request)
+                    .await
+                    .unwrap(),
             )
             .unwrap()
             .to_string(),
