@@ -5,7 +5,7 @@ use crate::inference::types::pyo3_helpers::{
     content_block_chat_output_to_python, deserialize_from_pyobj, serialize_to_dict, uuid_to_python,
 };
 use crate::inference::types::stored_input::StoredInput;
-use crate::inference::types::Text;
+use crate::inference::types::{RequestMessage, ResolvedRequestMessage, Text};
 use crate::{
     config::Config,
     error::{Error, ErrorDetails},
@@ -43,6 +43,7 @@ pub struct SimpleStoredSampleInfo {
     pub episode_id: Option<Uuid>,
     pub inference_id: Option<Uuid>,
     pub output: Option<Vec<ContentBlockChatOutput>>,
+    pub stored_output: Option<StoredOutput>,
     pub dispreferred_outputs: Vec<Vec<ContentBlockChatOutput>>,
     pub tool_params: Option<ToolCallConfigDatabaseInsert>,
     pub output_schema: Option<Value>,
@@ -378,14 +379,15 @@ impl StoredSample for StoredInference {
                 input: example.input,
                 episode_id: Some(example.episode_id),
                 inference_id: Some(example.inference_id),
-                output: Some(example.output),
+                output: Some(example.output.clone()),
+                stored_output: Some(StoredOutput::Chat(example.output)),
                 dispreferred_outputs: example.dispreferred_outputs,
                 tool_params: Some(example.tool_params),
                 output_schema: None,
                 tags: example.tags,
             },
             StoredInference::Json(example) => {
-                let output = json_output_to_content_block_chat_output(example.output);
+                let output = json_output_to_content_block_chat_output(example.output.clone());
                 let dispreferred_outputs = example
                     .dispreferred_outputs
                     .into_iter()
@@ -397,6 +399,7 @@ impl StoredSample for StoredInference {
                     episode_id: Some(example.episode_id),
                     inference_id: Some(example.inference_id),
                     output: Some(output),
+                    stored_output: Some(StoredOutput::Json(example.output)),
                     dispreferred_outputs,
                     tool_params: None,
                     output_schema: Some(example.output_schema),
@@ -416,17 +419,68 @@ fn json_output_to_content_block_chat_output(
     }
 }
 
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[cfg_attr(test, derive(ts_rs::TS))]
+#[serde(untagged)]
+pub enum StoredOutput {
+    Chat(Vec<ContentBlockChatOutput>),
+    Json(JsonInferenceOutput),
+}
+
 /// Represents an inference that has been prepared for fine-tuning.
 /// This is constructed by rendering a StoredInference with a variant for messages
 /// and by resolving all network resources (e.g. images).
 #[cfg_attr(feature = "pyo3", pyclass(str))]
 #[cfg_attr(test, derive(ts_rs::TS))]
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
+#[cfg_attr(any(feature = "e2e_tests", test), derive(PartialEq))]
 pub struct RenderedSample {
     pub function_name: String,
     pub input: ModelInput,
-    pub stored_input: ResolvedInput,
+    pub stored_input: StoredInput,
     pub output: Option<Vec<ContentBlockChatOutput>>,
+    pub stored_output: Option<StoredOutput>,
+    pub dispreferred_outputs: Vec<Vec<ContentBlockChatOutput>>,
+    pub episode_id: Option<Uuid>,
+    pub inference_id: Option<Uuid>,
+    pub tool_params: Option<ToolCallConfigDatabaseInsert>,
+    pub output_schema: Option<Value>,
+    pub tags: HashMap<String, String>,
+}
+
+impl RenderedSample {
+    pub fn into_lazy_rendered_sample(self) -> LazyRenderedSample {
+        LazyRenderedSample {
+            function_name: self.function_name,
+            system_input: self.input.system,
+            messages: self
+                .input
+                .messages
+                .into_iter()
+                .map(ResolvedRequestMessage::into_request_message)
+                .collect(),
+            stored_input: self.stored_input,
+            output: self.output,
+            stored_output: self.stored_output,
+            dispreferred_outputs: self.dispreferred_outputs,
+            episode_id: self.episode_id,
+            inference_id: self.inference_id,
+            tool_params: self.tool_params,
+            output_schema: self.output_schema,
+            tags: self.tags,
+        }
+    }
+}
+
+/// Like `RenderedSample`, but holds `RequestMessage`s instead of `ResolvedRequestMessage`s
+pub struct LazyRenderedSample {
+    pub function_name: String,
+    pub system_input: Option<String>,
+    // This is a a `Vec<ResolvedRequestMessage>` in `RenderedSample`
+    pub messages: Vec<RequestMessage>,
+    pub stored_input: StoredInput,
+    pub output: Option<Vec<ContentBlockChatOutput>>,
+    pub stored_output: Option<StoredOutput>,
     pub dispreferred_outputs: Vec<Vec<ContentBlockChatOutput>>,
     pub episode_id: Option<Uuid>,
     pub inference_id: Option<Uuid>,
@@ -461,6 +515,24 @@ impl RenderedSample {
         }
     }
 
+    #[getter]
+    pub fn get_stored_output<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
+        if let Some(stored_output) = &self.stored_output {
+            match stored_output {
+                StoredOutput::Chat(output) => {
+                    let output = output
+                        .iter()
+                        .map(|x| content_block_chat_output_to_python(py, x.clone()))
+                        .collect::<PyResult<Vec<_>>>()?;
+                    PyList::new(py, output).map(Bound::into_any)
+                }
+                StoredOutput::Json(output) => Ok(output.clone().into_py_any(py)?.into_bound(py)),
+            }
+        } else {
+            Ok(py.None().into_bound(py))
+        }
+    }
+
     #[getter]
     pub fn get_dispreferred_outputs<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
         let dispreferred_outputs = self
@@ -511,7 +583,7 @@ impl RenderedSample {
     }
 
     #[getter]
-    pub fn get_stored_input(&self) -> ResolvedInput {
+    pub fn get_stored_input(&self) -> StoredInput {
         self.stored_input.clone()
     }
 }
@@ -528,7 +600,7 @@ impl std::fmt::Display for RenderedSample {
 /// `variants` should be a map from function name to variant name, i.e. what variant to use for a particular function
 /// as the stored inference is being rendered.
 /// This does not handle resolving network resources (e.g. images).
-fn render_model_input(
+async fn render_model_input(
     resolved_input: &ResolvedInput,
     function_name: &str,
     config: &Config,
@@ -558,8 +630,9 @@ fn render_model_input(
         resolved_input.system.as_ref(),
         &resolved_input.messages,
         &config.templates,
-        &chat_completion_config.templates,
+        chat_completion_config.templates(),
     )
+    .await
 }
 
 /// Render an impl StoredSample to a RenderedStoredInference.
@@ -567,7 +640,7 @@ fn render_model_input(
 /// as the inference example is being rendered.
 ///
 /// This does not handle resolving network resources (e.g. images).
-pub fn render_stored_sample<T: StoredSample>(
+pub async fn render_stored_sample<T: StoredSample>(
     stored_sample: T,
     resolved_input: ResolvedInput,
     config: &Config,
@@ -577,6 +650,7 @@ pub fn render_stored_sample<T: StoredSample>(
         function_name,
         input: _,
         output,
+        stored_output,
         dispreferred_outputs,
         tool_params,
         output_schema,
@@ -584,14 +658,15 @@ pub fn render_stored_sample<T: StoredSample>(
         inference_id,
         tags,
     } = stored_sample.owned_simple_info();
-    let model_input = render_model_input(&resolved_input, &function_name, config, variants)?;
+    let model_input = render_model_input(&resolved_input, &function_name, config, variants).await?;
     Ok(RenderedSample {
         function_name,
         episode_id,
         inference_id,
         input: model_input,
-        stored_input: resolved_input,
+        stored_input: resolved_input.into_stored_input(),
         output,
+        stored_output,
         dispreferred_outputs,
         tool_params,
         output_schema,
