@@ -13,6 +13,7 @@ use crate::inference::types::pyo3_helpers::serialize_to_dict;
 use crate::{
     error::{Error, ErrorDetails},
     jsonschema_util::{DynamicJSONSchema, StaticJSONSchema},
+    rate_limiting::{get_estimated_tokens, RateLimitedInputContent},
 };
 
 /* A Tool is a function that can be called by an LLM
@@ -296,8 +297,7 @@ pub struct DynamicToolParams {
     pub parallel_tool_calls: Option<bool>,
 }
 
-#[derive(Debug, Deserialize, PartialEq, Serialize)]
-#[cfg_attr(test, derive(Default))]
+#[derive(Debug, Default, Deserialize, PartialEq, Serialize)]
 pub struct BatchDynamicToolParams {
     pub allowed_tools: Option<Vec<Option<Vec<String>>>>,
     pub additional_tools: Option<Vec<Option<Vec<Tool>>>>,
@@ -326,6 +326,18 @@ impl std::fmt::Display for ToolCall {
     }
 }
 
+impl RateLimitedInputContent for ToolCall {
+    fn estimated_input_token_usage(&self) -> u64 {
+        let ToolCall {
+            name,
+            arguments,
+            #[expect(unused_variables)]
+            id,
+        } = self;
+        get_estimated_tokens(name) + get_estimated_tokens(arguments)
+    }
+}
+
 #[cfg(feature = "pyo3")]
 #[pymethods]
 impl ToolCall {
@@ -455,6 +467,7 @@ impl ToolCallOutput {
 
 impl ToolCallConfig {
     #[cfg(test)]
+    #[expect(clippy::missing_panics_doc)]
     pub fn implicit_from_value(value: &Value) -> Self {
         let parameters = StaticJSONSchema::from_value(value.clone()).unwrap();
         let implicit_tool_config = ToolConfig::Implicit(ImplicitToolConfig { parameters });
@@ -484,6 +497,18 @@ impl std::fmt::Display for ToolResult {
     }
 }
 
+impl RateLimitedInputContent for ToolResult {
+    fn estimated_input_token_usage(&self) -> u64 {
+        let ToolResult {
+            name,
+            result,
+            #[expect(unused_variables)]
+            id,
+        } = self;
+        get_estimated_tokens(name) + get_estimated_tokens(result)
+    }
+}
+
 #[cfg(feature = "pyo3")]
 #[pymethods]
 impl ToolResult {
