@@ -7,8 +7,9 @@ use std::time::Duration;
 
 use async_trait::async_trait;
 use paste::paste;
-use secrecy::{ExposeSecret, SecretString};
+use secrecy::ExposeSecret;
 use serde_json::json;
+use tensorzero_core::config::BatchWritesConfig;
 use tensorzero_core::db::clickhouse::migration_manager::migration_trait::Migration;
 use tensorzero_core::db::clickhouse::migration_manager::{
     RunMigrationArgs, RunMigrationManagerArgs,
@@ -37,9 +38,7 @@ use tensorzero_core::db::clickhouse::migration_manager::{
     self, get_all_migration_records, make_all_migrations, MigrationRecordDatabaseInsert,
 };
 use tensorzero_core::db::clickhouse::test_helpers::{get_clickhouse, CLICKHOUSE_URL};
-use tensorzero_core::db::clickhouse::{
-    make_clickhouse_http_client, ClickHouseConnectionInfo, Rows, TableName,
-};
+use tensorzero_core::db::clickhouse::{ClickHouseConnectionInfo, Rows, TableName};
 
 pub struct DeleteDbOnDrop {
     database: String,
@@ -80,7 +79,9 @@ impl Drop for DeleteDbOnDrop {
 /// happen even if the test panics).
 /// This helps to reduce peak disk usage on CI.
 /// If `allow_db_missing` is true, then we'll use 'DROP DATABASE IF EXISTS' instead of 'DROP DATABASE'
-pub fn get_clean_clickhouse(allow_db_missing: bool) -> (ClickHouseConnectionInfo, DeleteDbOnDrop) {
+pub async fn get_clean_clickhouse(
+    allow_db_missing: bool,
+) -> (ClickHouseConnectionInfo, DeleteDbOnDrop) {
     let database = format!(
         "tensorzero_e2e_tests_migration_manager_{}",
         Uuid::now_v7().simple()
@@ -88,15 +89,21 @@ pub fn get_clean_clickhouse(allow_db_missing: bool) -> (ClickHouseConnectionInfo
     let mut clickhouse_url = url::Url::parse(&CLICKHOUSE_URL).unwrap();
     clickhouse_url.set_path("");
     clickhouse_url.set_query(Some(format!("database={database}").as_str()));
-    let cluster_name = std::env::var("TENSORZERO_CLICKHOUSE_CLUSTER_NAME").ok();
+    let clickhouse_url = clickhouse_url.to_string();
+    // Set TENSORZERO_E2E_TESTS_DATABASE so the client can use it
+    std::env::set_var("TENSORZERO_E2E_TESTS_DATABASE", &database);
+    let clickhouse = ClickHouseConnectionInfo::new(
+        &clickhouse_url,
+        BatchWritesConfig {
+            enabled: false,
+            __force_allow_embedded_batch_writes: false,
+            flush_interval_ms: 1000,
+            max_rows: 100,
+        },
+    )
+    .await
+    .unwrap();
 
-    let clickhouse = ClickHouseConnectionInfo::Production {
-        database_url: SecretString::from(clickhouse_url.to_string()),
-        database: database.clone(),
-        cluster_name,
-        client: make_clickhouse_http_client().unwrap(),
-        batch_sender: None,
-    };
     (
         clickhouse.clone(),
         DeleteDbOnDrop {
@@ -174,7 +181,8 @@ async fn count_table_rows(clickhouse: &ClickHouseConnectionInfo, table: &str) ->
 
 async fn insert_large_fixtures(clickhouse: &ClickHouseConnectionInfo) {
     // Insert data so that we test the migration re-creates the tables properly.
-    let s3_fixtures_path = format!("{MANIFEST_PATH}/../ui/fixtures/s3-fixtures");
+    let s3_fixtures_path = std::env::var("TENSORZERO_S3_FIXTURES_PATH")
+        .unwrap_or_else(|_| format!("{MANIFEST_PATH}/../ui/fixtures/s3-fixtures"));
     let s3_fixtures_path = &s3_fixtures_path;
 
     let ClickHouseConnectionInfo::Production {
@@ -229,29 +237,51 @@ async fn insert_large_fixtures(clickhouse: &ClickHouseConnectionInfo) {
     .map(|(file, table)| {
         let password = password.clone();
         async move {
-            let mut command = tokio::process::Command::new("docker");
-            command.args([
-                "run",
-                "--add-host=host.docker.internal:host-gateway",
-                "-v",
-                &format!("{s3_fixtures_path}:/s3-fixtures"),
-                "clickhouse/clickhouse-server:25.4-alpine",
-                "clickhouse-client",
-                "--host",
-                host,
-                "--user",
-                username,
-                "--password",
-                &password,
-                "--database",
-                database,
-                "--query",
-                &format!(
-                    r"
+            // If we are running in CI (TENSORZERO_CI=1), we should have the clickhouse client installed locally
+            // so we should not use Docker
+            let mut command = if std::env::var("TENSORZERO_CI").is_ok() {
+                let mut cmd = tokio::process::Command::new("clickhouse-client");
+                cmd.args([
+                    "--host",
+                    host,
+                    "--user",
+                    username,
+                    "--password",
+                    &password,
+                    "--database",
+                    database,
+                    "--query",
+                    &format!("INSERT INTO {table} SELECT * FROM file('{file}', 'Parquet')"),
+                ]);
+                cmd
+            } else {
+                // If we are running locally, we should use docker so that we can
+                // be platform independent in how we insert these files into ClickHouse.
+                let mut cmd = tokio::process::Command::new("docker");
+                cmd.args([
+                    "run",
+                    "--add-host=host.docker.internal:host-gateway",
+                    "-v",
+                    &format!("{s3_fixtures_path}:/s3-fixtures"),
+                    "clickhouse/clickhouse-server:25.4-alpine",
+                    "clickhouse-client",
+                    "--host",
+                    host,
+                    "--user",
+                    username,
+                    "--password",
+                    &password,
+                    "--database",
+                    database,
+                    "--query",
+                    &format!(
+                        r"
         INSERT INTO {table} FROM INFILE '/s3-fixtures/{file}' FORMAT Parquet
     "
-                ),
-            ]);
+                    ),
+                ]);
+                cmd
+            };
             assert!(
                 command.spawn().unwrap().wait().await.unwrap().success(),
                 "Failed to insert {table}"
@@ -369,8 +399,6 @@ async fn run_migration_0020_with_data<R: Future<Output = bool>, F: FnOnce() -> R
     let sample_chat_id = sample_chat_row_json["id_uint"].as_str().unwrap();
     let sample_chat_episode_id = sample_chat_row_json["episode_id_uint"].as_str().unwrap();
 
-    println!("Querying for sample chat by id: {sample_chat_id}");
-
     let matching_chat_by_id = clickhouse
         .run_query_synchronous_no_params(
             format!("SELECT id_uint, toUInt128(episode_id) as episode_id_uint FROM InferenceById WHERE function_type = 'chat' AND id_uint = '{sample_chat_id}' LIMIT 1 FORMAT JSONEachRow"),
@@ -469,7 +497,7 @@ async fn run_rollback_instructions(
     }
 }
 async fn test_rollback_helper(migration_num: usize, logs_contain: fn(&str) -> bool) {
-    let (fresh_clickhouse, _cleanup_fresh_clickhouse) = get_clean_clickhouse(true);
+    let (fresh_clickhouse, _cleanup_fresh_clickhouse) = get_clean_clickhouse(true).await;
     fresh_clickhouse
         .create_database_and_migrations_table()
         .await
@@ -522,14 +550,14 @@ invoke_all_separate_tests!(
     test_rollback_up_to_migration_index_,
     [
         0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
-        25, 26, 27, 28, 29
+        25, 26, 27, 28, 29, 30, 31
     ]
 );
 
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 async fn test_rollback_apply_rollback() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     clickhouse
         .create_database_and_migrations_table()
         .await
@@ -590,7 +618,7 @@ async fn test_rollback_apply_rollback() {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn test_clickhouse_migration_manager() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     clickhouse
         .create_database_and_migrations_table()
         .await
@@ -747,7 +775,7 @@ async fn test_clickhouse_migration_manager() {
         // for each element in the array.
         [
             0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
-            24, 25, 26, 27, 28, 29
+            24, 25, 26, 27, 28, 29, 30, 31
         ]
     );
     let rows = get_all_migration_records(&clickhouse).await.unwrap();
@@ -834,6 +862,14 @@ async fn test_clickhouse_migration_manager() {
     let output_token_total: u64 = response.response.trim().parse().unwrap();
     assert_eq!(output_token_total, 200000000);
 
+    // Check that the EpisodeById migration worked
+    let response = clickhouse
+        .run_query_synchronous_no_params("SELECT count() FROM EpisodeById".to_string())
+        .await
+        .unwrap();
+    let episode_count: u64 = response.response.trim().parse().unwrap();
+    assert_eq!(episode_count, 20000000);
+
     // Since we've already ran all of the migrations, we shouldn't have written any new records
     // except for Migration0029 (which runs every time)
 
@@ -872,7 +908,7 @@ async fn test_bad_clickhouse_write() {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn test_clean_clickhouse_start() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     let database = clickhouse.database();
     let is_manual = clickhouse.is_cluster_configured();
     migration_manager::run(RunMigrationManagerArgs {
@@ -921,7 +957,7 @@ async fn test_clean_clickhouse_start() {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn test_startup_without_migration_table() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     let is_manual = clickhouse.is_cluster_configured();
     // Run the migrations so we can get the database into a "dirty" state
     migration_manager::run(RunMigrationManagerArgs {
@@ -960,7 +996,7 @@ async fn test_startup_without_migration_table() {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn test_deployment_id_oldest() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     migration_manager::run(RunMigrationManagerArgs {
         clickhouse: &clickhouse,
         manual_run: true,
@@ -998,7 +1034,7 @@ async fn test_concurrent_clickhouse_migrations() {
         // We can't run concurrent migrations on a cluster.
         return;
     }
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     let clickhouse = Arc::new(clickhouse);
     let num_concurrent_starts = 50;
     let mut handles = Vec::with_capacity(num_concurrent_starts);
@@ -1048,7 +1084,7 @@ async fn test_concurrent_clickhouse_migrations() {
 /// rather than brick the database.
 #[tokio::test(flavor = "multi_thread")]
 async fn test_migration_0013_old_table() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     clickhouse
         .create_database_and_migrations_table()
         .await
@@ -1137,7 +1173,7 @@ async fn test_migration_0013_old_table() {
 /// This should fail.
 #[tokio::test(flavor = "multi_thread")]
 async fn test_migration_0013_data_no_table() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     clickhouse
         .create_database_and_migrations_table()
         .await
@@ -1216,7 +1252,7 @@ async fn test_migration_0013_data_no_table() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 async fn test_run_migrations_clean() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     migration_manager::run(RunMigrationManagerArgs {
         clickhouse: &clickhouse,
         manual_run: true,
@@ -1241,7 +1277,7 @@ async fn test_run_migrations_clean() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 async fn test_run_migrations_fake_row() {
-    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false);
+    let (clickhouse, _cleanup_db) = get_clean_clickhouse(false).await;
     clickhouse
         .create_database_and_migrations_table()
         .await
@@ -1277,9 +1313,10 @@ async fn test_run_migrations_fake_row() {
         skip_completed_migrations: true,
     })
     .await;
-    if migration_manager_result.is_err() ^ clickhouse.is_cluster_configured() {
-        panic!("Migration manager should fail to run if and only if a cluster is configured");
-    }
+    assert!(
+        !(migration_manager_result.is_err() ^ clickhouse.is_cluster_configured()),
+        "Migration manager should fail to run if and only if a cluster is configured"
+    );
 
     // Run our fake migration to insert an unexpected row into `TensorZeroMigration`
     // A subsequent normal run of migrations should *not* skip running migrations,
@@ -1303,9 +1340,10 @@ async fn test_run_migrations_fake_row() {
         manual_run: false,
     })
     .await;
-    if migration_manager_result.is_err() ^ clickhouse.is_cluster_configured() {
-        panic!("Migration manager should fail to run if and only if a cluster is configured");
-    }
+    assert!(
+        !(migration_manager_result.is_err() ^ clickhouse.is_cluster_configured()),
+        "Migration manager should fail to run if and only if a cluster is configured"
+    );
     assert!(!logs_contain("already been applied"));
 
     let rows = migration_manager::get_all_migration_records(&clickhouse)
