@@ -2,17 +2,20 @@ use reqwest::{Client, StatusCode};
 use serde_json::{json, Value};
 use tensorzero_core::{
     config::{Config, MetricConfig, MetricConfigLevel, MetricConfigOptimize, MetricConfigType},
-    db::clickhouse::test_helpers::{select_feedback_clickhouse, select_feedback_tags_clickhouse},
+    db::{
+        clickhouse::test_helpers::{select_feedback_clickhouse, select_feedback_tags_clickhouse},
+        postgres::PostgresConnectionInfo,
+    },
     endpoints::feedback::{feedback, Params},
-    gateway_util::GatewayHandle,
+    http::TensorzeroHttpClient,
     inference::types::{ContentBlockChatOutput, JsonInferenceOutput, Role, Text, TextKind},
+    utils::gateway::GatewayHandle,
 };
 use tokio::time::{sleep, Duration};
 use tracing_test::traced_test;
 use uuid::Uuid;
 
 use crate::common::get_gateway_endpoint;
-use crate::providers::common::make_embedded_gateway;
 use tensorzero_core::db::clickhouse::test_helpers::get_clickhouse;
 
 #[tokio::test]
@@ -178,10 +181,11 @@ async fn e2e_test_comment_feedback_validation_disabled() {
     let mut config = Config::default();
     let clickhouse = get_clickhouse().await;
     config.gateway.unstable_disable_feedback_target_validation = true;
-    let handle = GatewayHandle::new_with_clickhouse_and_http_client(
+    let handle = GatewayHandle::new_with_database_and_http_client(
         config.into(),
         clickhouse.clone(),
-        reqwest::Client::new(),
+        PostgresConnectionInfo::Disabled,
+        TensorzeroHttpClient::new().unwrap(),
     );
     let inference_id = Uuid::now_v7();
     let params = Params {
@@ -1209,10 +1213,11 @@ async fn e2e_test_float_feedback_validation_disabled() {
         .insert("user_score".to_string(), metric_config);
     let clickhouse = get_clickhouse().await;
     config.gateway.unstable_disable_feedback_target_validation = true;
-    let handle = GatewayHandle::new_with_clickhouse_and_http_client(
+    let handle = GatewayHandle::new_with_database_and_http_client(
         config.into(),
         clickhouse.clone(),
-        reqwest::Client::new(),
+        PostgresConnectionInfo::Disabled,
+        TensorzeroHttpClient::new().unwrap(),
     );
     let inference_id = Uuid::now_v7();
     let params = Params {
@@ -1443,10 +1448,11 @@ async fn e2e_test_boolean_feedback_validation_disabled() {
         .insert("task_success".to_string(), metric_config);
     let clickhouse = get_clickhouse().await;
     config.gateway.unstable_disable_feedback_target_validation = true;
-    let handle = GatewayHandle::new_with_clickhouse_and_http_client(
+    let handle = GatewayHandle::new_with_database_and_http_client(
         config.into(),
         clickhouse.clone(),
-        reqwest::Client::new(),
+        PostgresConnectionInfo::Disabled,
+        TensorzeroHttpClient::new().unwrap(),
     );
     let inference_id = Uuid::now_v7();
     let params = Params {
@@ -1478,7 +1484,7 @@ async fn test_fast_inference_then_feedback() {
     use std::collections::HashMap;
     use std::sync::Arc;
     // Create the client and wrap it in an Arc for shared ownership.
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
     let client = Arc::new(client);
 
     // Create a collection of tasks, each making an inference then a feedback call.
