@@ -10,16 +10,13 @@ use serde_json::{json, Value};
 use tracing_test::traced_test;
 use uuid::Uuid;
 
-use crate::{
-    common::get_gateway_endpoint,
-    providers::common::{make_embedded_gateway, make_embedded_gateway_no_config},
-};
+use crate::common::get_gateway_endpoint;
 use tensorzero_core::{
     db::clickhouse::test_helpers::{
         get_clickhouse, select_chat_inference_clickhouse, select_json_inference_clickhouse,
         select_model_inference_clickhouse,
     },
-    gateway_util::StructuredJson,
+    utils::gateway::StructuredJson,
 };
 
 #[tokio::test(flavor = "multi_thread")]
@@ -43,7 +40,7 @@ async fn test_openai_compatible_route_old_format() {
 }
 
 async fn test_openai_compatible_route_with_function_name_as_model(model: &str) {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
     let state = client.get_app_state_data().unwrap().clone();
     let episode_id = Uuid::now_v7();
 
@@ -885,7 +882,7 @@ async fn test_openai_compatible_streaming_tool_call() {
 #[tokio::test]
 #[traced_test]
 async fn test_openai_compatible_warn_headers() {
-    let client = make_embedded_gateway_no_config().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway_no_config().await;
     let state = client.get_app_state_data().unwrap().clone();
     let episode_id = Uuid::now_v7();
     let _ = tensorzero_core::endpoints::openai_compatible::inference_handler(
@@ -928,7 +925,7 @@ async fn test_openai_compatible_warn_headers() {
 #[tokio::test]
 #[traced_test]
 async fn test_openai_compatible_warn_unknown_fields() {
-    let client = make_embedded_gateway_no_config().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway_no_config().await;
     let state = client.get_app_state_data().unwrap().clone();
     tensorzero_core::endpoints::openai_compatible::inference_handler(
         State(state),
@@ -952,7 +949,7 @@ async fn test_openai_compatible_warn_unknown_fields() {
 
 #[tokio::test]
 async fn test_openai_compatible_deny_unknown_fields() {
-    let client = make_embedded_gateway_no_config().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway_no_config().await;
     let state = client.get_app_state_data().unwrap().clone();
     let err = tensorzero_core::endpoints::openai_compatible::inference_handler(
         State(state),
