@@ -11,21 +11,18 @@ use tensorzero_core::inference::types::stored_input::StoredFile;
 use tensorzero_core::inference::types::stored_input::{
     StoredInput, StoredInputMessage, StoredInputMessageContent,
 };
+use tensorzero_core::inference::types::{ResolvedContentBlock, ResolvedRequestMessage};
 use tensorzero_core::{
-    inference::types::{
-        ContentBlock, ContentBlockChatOutput, JsonInferenceOutput, RequestMessage, Text,
-    },
+    inference::types::{ContentBlockChatOutput, JsonInferenceOutput, Text},
     tool::{ToolCallConfigDatabaseInsert, ToolCallOutput, ToolChoice},
 };
 use tracing_test::traced_test;
 use uuid::Uuid;
 
-use crate::providers::common::make_embedded_gateway;
-
 /// Test that the render_samples function works when given an empty array of stored inferences.
 #[tokio::test(flavor = "multi_thread")]
 pub async fn test_render_samples_empty() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     // Test with an empty stored inferences array.
     let stored_inferences: Vec<StoredInference> = vec![];
@@ -41,7 +38,7 @@ pub async fn test_render_samples_empty() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_samples_no_function() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
         function_name: "basic_test".to_string(),
@@ -77,7 +74,7 @@ pub async fn test_render_samples_no_function() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_samples_no_variant() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
         function_name: "basic_test".to_string(),
@@ -121,7 +118,7 @@ pub async fn test_render_samples_no_variant() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_samples_missing_variable() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
         function_name: "basic_test".to_string(),
@@ -159,7 +156,7 @@ pub async fn test_render_samples_missing_variable() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_samples_normal() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let stored_inferences = vec![
         StoredInference::Chat(StoredChatInference {
@@ -311,7 +308,7 @@ pub async fn test_render_samples_normal() {
     assert_eq!(first_message.role, Role::User);
     assert_eq!(first_message.content.len(), 1);
 
-    let ContentBlock::Text(text) = &first_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &first_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "Hello, world!");
@@ -336,7 +333,7 @@ pub async fn test_render_samples_normal() {
     assert_eq!(second_message.role, Role::User);
     assert_eq!(second_message.content.len(), 1);
 
-    let ContentBlock::Text(text) = &second_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &second_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "What is the name of the capital city of Japan?");
@@ -375,7 +372,7 @@ pub async fn test_render_samples_normal() {
     assert_eq!(third_message.role, Role::User);
     assert_eq!(third_message.content.len(), 1);
 
-    let ContentBlock::Text(text) = &third_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &third_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "Hello, world!");
@@ -417,12 +414,12 @@ pub async fn test_render_samples_normal() {
     assert_eq!(fourth_message.role, Role::User);
     assert_eq!(fourth_message.content.len(), 2);
 
-    let ContentBlock::Text(text) = &fourth_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &fourth_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "What is this a picture of?");
 
-    let ContentBlock::File(file) = &fourth_message.content[1] else {
+    let ResolvedContentBlock::File(file) = &fourth_message.content[1] else {
         panic!("Expected file content");
     };
 
@@ -440,7 +437,7 @@ pub async fn test_render_samples_normal() {
 /// Test that the render_samples function can render a normal chat example, a tool call example, a json example, and an example using images.
 #[tokio::test(flavor = "multi_thread")]
 pub async fn test_render_samples_template_no_schema() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
         function_name: "basic_test_template_no_schema".to_string(),
@@ -504,13 +501,13 @@ pub async fn test_render_samples_template_no_schema() {
 
     assert_eq!(
         first_inference.input.messages[0],
-        RequestMessage {
+        ResolvedRequestMessage {
             role: Role::User,
             content: vec![
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "User content: `First user message`".into(),
                 }),
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "User content: `Second user message`".into(),
                 })
             ],
@@ -519,13 +516,13 @@ pub async fn test_render_samples_template_no_schema() {
 
     assert_eq!(
         first_inference.input.messages[1],
-        RequestMessage {
+        ResolvedRequestMessage {
             role: Role::Assistant,
             content: vec![
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "Assistant content: `First assistant message`".into(),
                 }),
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "Assistant content: `Second assistant message`".into(),
                 })
             ],
@@ -544,7 +541,7 @@ pub async fn test_render_samples_template_no_schema() {
 /// Test that the render_samples function works when given an empty array of datapoints.
 #[tokio::test(flavor = "multi_thread")]
 pub async fn test_render_datapoints_empty() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     // Test with an empty datapoints array.
     let datapoints: Vec<Datapoint> = vec![];
@@ -560,7 +557,7 @@ pub async fn test_render_datapoints_empty() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_datapoints_no_function() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
@@ -599,7 +596,7 @@ pub async fn test_render_datapoints_no_function() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_datapoints_no_variant() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
@@ -646,7 +643,7 @@ pub async fn test_render_datapoints_no_variant() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_datapoints_missing_variable() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
@@ -687,7 +684,7 @@ pub async fn test_render_datapoints_missing_variable() {
 #[tokio::test(flavor = "multi_thread")]
 #[traced_test]
 pub async fn test_render_datapoints_normal() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let datapoints = vec![
         Datapoint::Chat(ChatInferenceDatapoint {
@@ -846,7 +843,7 @@ pub async fn test_render_datapoints_normal() {
     assert_eq!(first_message.role, Role::User);
     assert_eq!(first_message.content.len(), 1);
 
-    let ContentBlock::Text(text) = &first_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &first_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "Hello, world!");
@@ -871,7 +868,7 @@ pub async fn test_render_datapoints_normal() {
     assert_eq!(second_message.role, Role::User);
     assert_eq!(second_message.content.len(), 1);
 
-    let ContentBlock::Text(text) = &second_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &second_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "What is the name of the capital city of Japan?");
@@ -903,7 +900,7 @@ pub async fn test_render_datapoints_normal() {
     assert_eq!(third_message.role, Role::User);
     assert_eq!(third_message.content.len(), 1);
 
-    let ContentBlock::Text(text) = &third_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &third_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "Hello, world!");
@@ -938,12 +935,12 @@ pub async fn test_render_datapoints_normal() {
     assert_eq!(fourth_message.role, Role::User);
     assert_eq!(fourth_message.content.len(), 2);
 
-    let ContentBlock::Text(text) = &fourth_message.content[0] else {
+    let ResolvedContentBlock::Text(text) = &fourth_message.content[0] else {
         panic!("Expected text content");
     };
     assert_eq!(text.text, "What is this a picture of?");
 
-    let ContentBlock::File(file) = &fourth_message.content[1] else {
+    let ResolvedContentBlock::File(file) = &fourth_message.content[1] else {
         panic!("Expected file content");
     };
 
@@ -961,7 +958,7 @@ pub async fn test_render_datapoints_normal() {
 /// Test that the render_samples function can render a datapoint with template but no schema.
 #[tokio::test(flavor = "multi_thread")]
 pub async fn test_render_datapoints_template_no_schema() {
-    let client = make_embedded_gateway().await;
+    let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
@@ -1028,13 +1025,13 @@ pub async fn test_render_datapoints_template_no_schema() {
 
     assert_eq!(
         first_sample.input.messages[0],
-        RequestMessage {
+        ResolvedRequestMessage {
             role: Role::User,
             content: vec![
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "User content: `First user message`".into(),
                 }),
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "User content: `Second user message`".into(),
                 })
             ],
@@ -1043,13 +1040,13 @@ pub async fn test_render_datapoints_template_no_schema() {
 
     assert_eq!(
         first_sample.input.messages[1],
-        RequestMessage {
+        ResolvedRequestMessage {
             role: Role::Assistant,
             content: vec![
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "Assistant content: `First assistant message`".into(),
                 }),
-                ContentBlock::Text(Text {
+                ResolvedContentBlock::Text(Text {
                     text: "Assistant content: `Second assistant message`".into(),
                 })
             ],
