@@ -1,4 +1,4 @@
-#![expect(clippy::panic, clippy::unwrap_used, clippy::expect_used)]
+#![expect(clippy::expect_used, clippy::panic, clippy::unwrap_used)]
 mod batch;
 mod best_of_n;
 mod cache;
@@ -6,6 +6,7 @@ mod clickhouse;
 mod common;
 mod config;
 mod datasets;
+mod db;
 mod dicl;
 mod dynamic_evaluations;
 mod dynamic_variants;
@@ -22,10 +23,11 @@ mod object_storage;
 mod openai_compatible;
 mod optimization;
 mod otel;
-mod otel_jaeger;
+mod otel_export;
 mod prometheus;
 mod providers;
 mod proxy;
+mod rate_limiting;
 mod render_inferences;
 mod retries;
 mod streaming_errors;
