@@ -1,4 +1,4 @@
-use crate::common::OptimizationTestCase;
+use crate::common::{mock_inference_provider_base, OptimizationTestCase};
 use tensorzero_core::{
     model::CredentialLocation,
     optimization::{
@@ -32,7 +32,11 @@ impl OptimizationTestCase for GCPVertexGeminiSFTTestCase {
                     credentials: Some(CredentialLocation::Sdk),
                     seed: None,
                     api_base: if use_mock_inference_provider {
-                        Some("http://localhost:3030/gcp_vertex_gemini/".parse().unwrap())
+                        Some(
+                            mock_inference_provider_base()
+                                .join("gcp_vertex_gemini/")
+                                .unwrap(),
+                        )
                     } else {
                         None
                     },
