@@ -1,30 +1,36 @@
-#![expect(clippy::unwrap_used, clippy::panic, clippy::print_stdout)]
+#![expect(clippy::panic, clippy::print_stdout, clippy::unwrap_used)]
 use base64::Engine;
 use serde_json::json;
 use std::collections::HashMap;
 use tokio::time::{sleep, Duration};
+use url::Url;
 use uuid::Uuid;
 
 use tracing_subscriber::{self, EnvFilter};
 
-use tensorzero::{
-    Client, InferenceOutputSource, LaunchOptimizationWorkflowParams, RenderedSample, Role,
-};
+use tensorzero::{InferenceOutputSource, LaunchOptimizationWorkflowParams, RenderedSample, Role};
 use tensorzero_core::{
     cache::CacheOptions,
     config::{Config, ConfigFileGlob, ProviderTypesConfig},
-    db::clickhouse::test_helpers::{get_clickhouse, CLICKHOUSE_URL},
-    db::clickhouse::{ClickHouseConnectionInfo, ClickhouseFormat},
+    db::{
+        clickhouse::{test_helpers::CLICKHOUSE_URL, ClickHouseConnectionInfo, ClickhouseFormat},
+        postgres::PostgresConnectionInfo,
+    },
     endpoints::inference::InferenceClients,
+    http::TensorzeroHttpClient,
     inference::types::{
+        file::Base64FileMetadata,
         resolved_input::FileWithPath,
         storage::{StorageKind, StoragePath},
+        stored_input::StoredFile,
         Base64File, ContentBlock, ContentBlockChatOutput, FunctionType, ModelInferenceRequest,
-        ModelInput, RequestMessage, ResolvedInput, ResolvedInputMessage,
-        ResolvedInputMessageContent, Text,
+        ModelInput, RequestMessage, ResolvedContentBlock, ResolvedRequestMessage, StoredInput,
+        StoredInputMessage, StoredInputMessageContent, Text,
     },
-    optimization::JobHandle,
-    optimization::{OptimizationJobInfo, Optimizer, OptimizerOutput, UninitializedOptimizerInfo},
+    optimization::{
+        JobHandle, OptimizationJobInfo, Optimizer, OptimizerOutput, UninitializedOptimizerInfo,
+    },
+    stored_inference::StoredOutput,
     tool::{Tool, ToolCall, ToolCallConfigDatabaseInsert, ToolCallOutput, ToolChoice, ToolResult},
     variant::JsonMode,
 };
@@ -32,6 +38,7 @@ use tensorzero_core::{
 pub mod dicl;
 pub mod fireworks_sft;
 pub mod gcp_vertex_gemini_sft;
+pub mod openai_rft;
 pub mod openai_sft;
 pub mod together_sft;
 
@@ -41,6 +48,13 @@ fn use_mock_inference_provider() -> bool {
     std::env::var("TENSORZERO_USE_MOCK_INFERENCE_PROVIDER").is_ok()
 }
 
+pub fn mock_inference_provider_base() -> Url {
+    std::env::var("TENSORZERO_MOCK_INFERENCE_PROVIDER_BASE_URL")
+        .unwrap_or_else(|_| "http://localhost:3030/".to_string())
+        .parse()
+        .unwrap()
+}
+
 pub trait OptimizationTestCase {
     fn supports_image_data(&self) -> bool;
     fn supports_tool_calls(&self) -> bool;
@@ -59,13 +73,35 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
         .load()
         .await
         .unwrap();
-    let client = reqwest::Client::new();
+
+    let client = TensorzeroHttpClient::new().unwrap();
     let test_examples = get_examples(test_case, 10);
     let val_examples = Some(get_examples(test_case, 10));
     let credentials: HashMap<String, secrecy::SecretBox<str>> = HashMap::new();
-    let clickhouse = get_clickhouse().await;
+
     let mut config_path = std::path::PathBuf::from(env!("CARGO_MANIFEST_DIR"));
     config_path.push("tests/e2e/tensorzero.toml");
+
+    // Create an embedded client so that we run migrations
+    let tensorzero_client =
+        tensorzero::ClientBuilder::new(tensorzero::ClientBuilderMode::EmbeddedGateway {
+            config_file: Some(config_path.clone()),
+            clickhouse_url: Some(CLICKHOUSE_URL.clone()),
+            postgres_url: None,
+            timeout: None,
+            verify_credentials: true,
+            allow_batch_writes: true,
+        })
+        .build()
+        .await
+        .unwrap();
+
+    let clickhouse = tensorzero_client
+        .get_app_state_data()
+        .unwrap()
+        .clickhouse_connection_info
+        .clone();
+
     let config_glob = ConfigFileGlob::new_from_path(&config_path).unwrap();
     let config = Config::load_from_path_optional_verify_credentials(
         &config_glob,
@@ -106,7 +142,7 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
         panic!("Expected completed status");
     };
 
-    // Handle both Model and Variant outputs
+    // Handle Model output only
     match output {
         OptimizerOutput::Model(model_config) => {
             let model_config = model_config
@@ -147,8 +183,12 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
             let clients = InferenceClients {
                 http_client: &client,
                 clickhouse_connection_info: &ClickHouseConnectionInfo::Disabled,
+                postgres_connection_info: &PostgresConnectionInfo::Disabled,
                 credentials: &HashMap::new(),
                 cache_options: &CacheOptions::default(),
+                tags: &Default::default(),
+                rate_limiting_config: &Default::default(),
+                otlp_config: &Default::default(),
             };
             // We didn't produce a real model, so there's nothing to test
             if use_mock_inference_provider() {
@@ -160,11 +200,8 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
                 .unwrap();
             println!("Response: {response:?}");
         }
-        OptimizerOutput::Variant(variant_config) => {
-            // Test the variant configuration
-            println!("Variant configuration created successfully: {variant_config:?}");
-            // For DICL variants, we don't need to test inference like we do for models
-            // since DICL variants work by retrieving examples at inference time
+        OptimizerOutput::Variant(_) => {
+            panic!("Expected model output, got variant output");
         }
     };
 }
@@ -213,6 +250,7 @@ pub async fn run_workflow_test_case_with_tensorzero_client(
 fn get_examples(test_case: &impl OptimizationTestCase, num_examples: usize) -> Vec<RenderedSample> {
     assert!(num_examples >= 10);
     let mut generators: Vec<fn() -> RenderedSample> = vec![generate_text_example];
+
     if test_case.supports_tool_calls() {
         generators.push(generate_tool_call_example);
     }
@@ -232,29 +270,31 @@ fn generate_text_example() -> RenderedSample {
     let id = Uuid::now_v7().to_string();
     let system_prompt =
         format!("You are a helpful assistant named Dr. M.M. Patel with id number {id}.");
+    let output = vec![ContentBlockChatOutput::Text(Text {
+        text: "The capital of France is Paris.".to_string(),
+    })];
     RenderedSample {
-        function_name: "test".to_string(),
+        function_name: "basic_test".to_string(),
         input: ModelInput {
             system: Some(system_prompt.clone()),
-            messages: vec![RequestMessage {
+            messages: vec![ResolvedRequestMessage {
                 role: Role::User,
-                content: vec![ContentBlock::Text(Text {
+                content: vec![ResolvedContentBlock::Text(Text {
                     text: "What is the capital of France?".to_string(),
                 })],
             }],
         },
-        stored_input: ResolvedInput {
+        stored_input: StoredInput {
             system: Some(json!(system_prompt)),
-            messages: vec![ResolvedInputMessage {
+            messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![ResolvedInputMessageContent::Text {
-                    value: json!("What is the capital of France?"),
+                content: vec![StoredInputMessageContent::Text {
+                    value: "What is the capital of France?".into(),
                 }],
             }],
         },
-        output: Some(vec![ContentBlockChatOutput::Text(Text {
-            text: "The capital of France is Paris.".to_string(),
-        })]),
+        output: Some(output.clone()),
+        stored_output: Some(StoredOutput::Chat(output)),
         episode_id: Some(Uuid::now_v7()),
         inference_id: Some(Uuid::now_v7()),
         tool_params: None,
@@ -271,24 +311,36 @@ fn generate_tool_call_example() -> RenderedSample {
     let id = Uuid::now_v7().to_string();
     let system_prompt =
         format!("You are a helpful assistant named Dr. M.M. Patel with id number {id}.");
+    let tool_call_output = vec![ContentBlockChatOutput::ToolCall(ToolCallOutput {
+        name: Some("get_weather".to_string()),
+        arguments: Some(serde_json::json!({
+            "location": "London",
+        })),
+        raw_name: "get_weather".to_string(),
+        raw_arguments: serde_json::json!({
+            "location": "London",
+        })
+        .to_string(),
+        id: "call_2".to_string(),
+    })];
     RenderedSample {
-        function_name: "test".to_string(),
+        function_name: "basic_test".to_string(),
         input: ModelInput {
             system: Some(system_prompt.clone()),
             messages: vec![
-                RequestMessage {
+                ResolvedRequestMessage {
                     role: Role::User,
-                    content: vec![ContentBlock::Text(Text {
+                    content: vec![ResolvedContentBlock::Text(Text {
                         text: "What is the weather in Paris?".to_string(),
                     })],
                 },
-                RequestMessage {
+                ResolvedRequestMessage {
                     role: Role::Assistant,
                     content: vec![
-                        ContentBlock::Text(Text {
+                        ResolvedContentBlock::Text(Text {
                             text: "Let me look that up for you.".to_string(),
                         }),
-                        ContentBlock::ToolCall(ToolCall {
+                        ResolvedContentBlock::ToolCall(ToolCall {
                             name: "get_weather".to_string(),
                             arguments: serde_json::json!({
                                 "location": "Paris"
@@ -298,9 +350,9 @@ fn generate_tool_call_example() -> RenderedSample {
                         }),
                     ],
                 },
-                RequestMessage {
+                ResolvedRequestMessage {
                     role: Role::User,
-                    content: vec![ContentBlock::ToolResult(ToolResult {
+                    content: vec![ResolvedContentBlock::ToolResult(ToolResult {
                         name: "get_weather".to_string(),
                         result: serde_json::json!({
                             "weather": "sunny, 25 degrees Celsius",
@@ -309,36 +361,36 @@ fn generate_tool_call_example() -> RenderedSample {
                         id: "call_1".to_string(),
                     })],
                 },
-                RequestMessage {
+                ResolvedRequestMessage {
                     role: Role::Assistant,
-                    content: vec![ContentBlock::Text(Text {
+                    content: vec![ResolvedContentBlock::Text(Text {
                         text: "The weather in Paris is sunny, 25 degrees Celsius.".to_string(),
                     })],
                 },
-                RequestMessage {
+                ResolvedRequestMessage {
                     role: Role::User,
-                    content: vec![ContentBlock::Text(Text {
+                    content: vec![ResolvedContentBlock::Text(Text {
                         text: "What is the weather in London?".to_string(),
                     })],
                 },
             ],
         },
-        stored_input: ResolvedInput {
+        stored_input: StoredInput {
             system: Some(json!(system_prompt)),
             messages: vec![
-                ResolvedInputMessage {
+                StoredInputMessage {
                     role: Role::User,
-                    content: vec![ResolvedInputMessageContent::Text {
+                    content: vec![StoredInputMessageContent::Text {
                         value: json!("What is the weather in Paris?"),
                     }],
                 },
-                ResolvedInputMessage {
+                StoredInputMessage {
                     role: Role::Assistant,
                     content: vec![
-                        ResolvedInputMessageContent::Text {
+                        StoredInputMessageContent::Text {
                             value: json!("Let me look that up for you."),
                         },
-                        ResolvedInputMessageContent::ToolCall(ToolCall {
+                        StoredInputMessageContent::ToolCall(ToolCall {
                             name: "get_weather".to_string(),
                             arguments: serde_json::json!({
                                 "location": "Paris"
@@ -348,9 +400,9 @@ fn generate_tool_call_example() -> RenderedSample {
                         }),
                     ],
                 },
-                ResolvedInputMessage {
+                StoredInputMessage {
                     role: Role::User,
-                    content: vec![ResolvedInputMessageContent::ToolResult(ToolResult {
+                    content: vec![StoredInputMessageContent::ToolResult(ToolResult {
                         name: "get_weather".to_string(),
                         result: serde_json::json!({
                             "weather": "sunny, 25 degrees Celsius",
@@ -359,32 +411,22 @@ fn generate_tool_call_example() -> RenderedSample {
                         id: "call_1".to_string(),
                     })],
                 },
-                ResolvedInputMessage {
+                StoredInputMessage {
                     role: Role::Assistant,
-                    content: vec![ResolvedInputMessageContent::Text {
+                    content: vec![StoredInputMessageContent::Text {
                         value: json!("The weather in Paris is sunny, 25 degrees Celsius."),
                     }],
                 },
-                ResolvedInputMessage {
+                StoredInputMessage {
                     role: Role::User,
-                    content: vec![ResolvedInputMessageContent::Text {
+                    content: vec![StoredInputMessageContent::Text {
                         value: json!("What is the weather in London?"),
                     }],
                 },
             ],
         },
-        output: Some(vec![ContentBlockChatOutput::ToolCall(ToolCallOutput {
-            name: Some("get_weather".to_string()),
-            arguments: Some(serde_json::json!({
-                "location": "London",
-            })),
-            raw_name: "get_weather".to_string(),
-            raw_arguments: serde_json::json!({
-                "location": "London",
-            })
-            .to_string(),
-            id: "call_2".to_string(),
-        })]),
+        output: Some(tool_call_output.clone()),
+        stored_output: Some(StoredOutput::Chat(tool_call_output)),
         tool_params: Some(ToolCallConfigDatabaseInsert {
             tools_available: vec![Tool {
                 name: "get_weather".to_string(),
@@ -417,17 +459,20 @@ fn generate_image_example() -> RenderedSample {
     let id = Uuid::now_v7().to_string();
     let system_prompt =
         format!("You are a helpful assistant named Dr. M.M. Patel with id number {id}.");
+    let output = vec![ContentBlockChatOutput::Text(Text {
+        text: "Orange!".to_string(),
+    })];
     RenderedSample {
-        function_name: "test".to_string(),
+        function_name: "basic_test".to_string(),
         input: ModelInput {
             system: Some(system_prompt.clone()),
-            messages: vec![RequestMessage {
+            messages: vec![ResolvedRequestMessage {
                 role: Role::User,
                 content: vec![
-                    ContentBlock::Text(Text {
+                    ResolvedContentBlock::Text(Text {
                         text: "What is the main color of this image?".to_string(),
                     }),
-                    ContentBlock::File(Box::new(FileWithPath {
+                    ResolvedContentBlock::File(Box::new(FileWithPath {
                         file: Base64File {
                             url: None,
                             mime_type: mime::IMAGE_PNG,
@@ -443,19 +488,18 @@ fn generate_image_example() -> RenderedSample {
                 ],
             }],
         },
-        stored_input: ResolvedInput {
+        stored_input: StoredInput {
             system: Some(json!(system_prompt)),
-            messages: vec![ResolvedInputMessage {
+            messages: vec![StoredInputMessage {
                 role: Role::User,
                 content: vec![
-                    ResolvedInputMessageContent::Text {
+                    StoredInputMessageContent::Text {
                         value: json!("What is the main color of this image?"),
                     },
-                    ResolvedInputMessageContent::File(Box::new(FileWithPath {
-                        file: Base64File {
+                    StoredInputMessageContent::File(Box::new(StoredFile {
+                        file: Base64FileMetadata {
                             url: None,
                             mime_type: mime::IMAGE_PNG,
-                            data: base64::prelude::BASE64_STANDARD.encode(FERRIS_PNG),
                         },
                         storage_path: StoragePath {
                             kind: StorageKind::Disabled,
@@ -467,9 +511,8 @@ fn generate_image_example() -> RenderedSample {
                 ],
             }],
         },
-        output: Some(vec![ContentBlockChatOutput::Text(Text {
-            text: "Orange!".to_string(),
-        })]),
+        output: Some(output.clone()),
+        stored_output: Some(StoredOutput::Chat(output)),
         tool_params: None,
         episode_id: Some(Uuid::now_v7()),
         inference_id: Some(Uuid::now_v7()),
@@ -481,34 +524,6 @@ fn generate_image_example() -> RenderedSample {
     }
 }
 
-#[allow(clippy::allow_attributes, dead_code)]
-pub async fn make_embedded_gateway() -> Client {
-    let mut config_path = std::path::PathBuf::from(env!("CARGO_MANIFEST_DIR"));
-    config_path.push("tests/e2e/tensorzero.toml");
-    tensorzero::ClientBuilder::new(tensorzero::ClientBuilderMode::EmbeddedGateway {
-        config_file: Some(config_path),
-        clickhouse_url: Some(CLICKHOUSE_URL.clone()),
-        timeout: None,
-        verify_credentials: true,
-        allow_batch_writes: true,
-    })
-    .with_verbose_errors(true)
-    .build()
-    .await
-    .unwrap()
-}
-
-#[allow(clippy::allow_attributes, dead_code)]
-pub async fn make_http_gateway() -> Client {
-    tensorzero::ClientBuilder::new(tensorzero::ClientBuilderMode::HTTPGateway {
-        url: "http://localhost:3000".parse().unwrap(),
-    })
-    .with_verbose_errors(true)
-    .build()
-    .await
-    .unwrap()
-}
-
 /// Generates a `#[tokio::test] async fn $fn_name() { run_test_case(&$constructor).await; }`
 #[macro_export]
 macro_rules! optimization_test_case {
@@ -533,7 +548,7 @@ macro_rules! embedded_workflow_test_case {
         ::paste::paste! {
             #[tokio::test(flavor = "multi_thread")]
             async fn [<test_embedded_slow_optimization_ $fn_name>]() {
-                let client = $crate::common::make_embedded_gateway().await;
+                let client = tensorzero::test_helpers::make_embedded_gateway().await;
                 $crate::common::run_workflow_test_case_with_tensorzero_client(&$constructor, &client).await;
             }
         }
@@ -549,7 +564,7 @@ macro_rules! http_workflow_test_case {
         ::paste::paste! {
             #[tokio::test]
             async fn [<test_http_slow_optimization_ $fn_name>]() {
-                let client = $crate::common::make_http_gateway().await;
+                let client = tensorzero::test_helpers::make_http_gateway().await;
                 $crate::common::run_workflow_test_case_with_tensorzero_client(&$constructor, &client).await;
             }
         }
