@@ -27,6 +27,13 @@ class ContentBlock(ABC, HasTypeField):
     pass
 
 
+@dataclass
+class Template(ContentBlock):
+    name: str
+    arguments: Any
+    type: str = "template"
+
+
 @dataclass
 class Text(ContentBlock):
     text: Optional[str] = None
